# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.template import TemplateDoesNotExist
from django.template.loader import select_template
from django.utils.functional import lazy
from django.utils.safestring import mark_safe

import six


def concat_classes(classes):
    """
    merges a list of classes and return concatinated string
    """
    return ' '.join(_class for _class in classes if _class)


def get_template_path(prefix, template, name):
    return 'djangocms_bootstrap4/{}/{}/{}.html'.format(prefix, template, name)


def get_plugin_template(instance, prefix, name, templates):
    template = getattr(instance, 'template', templates[0][0])
    template_path = get_template_path(prefix, template, name)

    try:
        select_template([template_path])
    except TemplateDoesNotExist:
        # TODO render a warning inside the template
        template_path = get_template_path(prefix, 'default', name)

    return template_path


# use mark_safe_lazy to delay the translation when using mark_safe
# otherwise they will not be added to /locale/
# https://docs.djangoproject.com/en/1.11/topics/i18n/translation/#other-uses-of-lazy-in-delayed-translations
mark_safe_lazy = lazy(mark_safe, six.text_type)



# -*- coding: utf-8 -*-
from django.test import TestCase

from djangocms_bootstrap4.contrib.bootstrap4_tabs.models import (
    Bootstrap4Tab, Bootstrap4TabItem,
)


class B4TabsModelTestCase(TestCase):

    def test_tab_instance(self):
        instance = Bootstrap4Tab.objects.create()
        self.assertEqual(str(instance), "1")
        self.assertEqual(instance.get_short_description(), "(nav-tabs)")
        instance.tab_alignment = "nav-fill"
        self.assertEqual(instance.get_short_description(), "(nav-tabs) .nav-fill")

    def test_tab_item_instance(self):
        instance = Bootstrap4TabItem.objects.create()
        self.assertEqual(str(instance), "1")
        self.assertEqual(instance.get_short_description(), "")



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.conf import settings
from django.utils.translation import ugettext_lazy as _


TAB_TYPE_CHOICES = (
    ('nav-tabs', _('Tabs')),
    ('nav-pills', _('Pills')),
)

TAB_ALIGNMENT_CHOICES = (
    ('nav-fill', _('Fill')),
    ('nav-justified', _('Justified')),
    ('justify-content-start', _('Justify start')),
    ('justify-content-center', _('Justify center')),
    ('justify-content-end', _('Justify end')),
    ('flex-column', _('Column')),
)

TAB_EFFECT_CHOICES = (
    ('fade', _('Fade')),
)

TAB_TEMPLATE_CHOICES = getattr(
    settings,
    'DJANGOCMS_BOOTSTRAP4_TAB_TEMPLATES',
    (
        ('default', _('Default')),
    ),
)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.utils.translation import ugettext_lazy as _

from cms.plugin_base import CMSPluginBase
from cms.plugin_pool import plugin_pool

from djangocms_bootstrap4.helpers import concat_classes

from .models import Bootstrap4Jumbotron


class Bootstrap4JumbotronPlugin(CMSPluginBase):
    """
    Components > "Jumbotron" Plugin
    https://getbootstrap.com/docs/4.0/components/jumbotron/
    """
    model = Bootstrap4Jumbotron
    name = _('Jumbotron')
    module = _('Bootstrap 4')
    render_template = 'djangocms_bootstrap4/jumbotron.html'
    change_form_template = 'djangocms_bootstrap4/admin/jumbotron.html'
    allow_children = True

    fieldsets = [
        (None, {
            'fields': (
                'fluid',
            )
        }),
        (_('Advanced settings'), {
            'classes': ('collapse',),
            'fields': (
                'tag_type',
                'attributes',
            )
        }),
    ]

    def render(self, context, instance, placeholder):
        link_classes = ['jumbotron']
        if instance.fluid:
            link_classes.append('jumbotron-fluid')

        classes = concat_classes(link_classes + [
            instance.attributes.get('class'),
        ])
        instance.attributes['class'] = classes

        return super(Bootstrap4JumbotronPlugin, self).render(
            context, instance, placeholder
        )


plugin_pool.register_plugin(Bootstrap4JumbotronPlugin)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.forms import BooleanField, IntegerField, models
from django.utils.translation import ugettext_lazy as _

from djangocms_bootstrap4.constants import DEVICE_SIZES

from .constants import GRID_SIZE
from .models import Bootstrap4GridColumn, Bootstrap4GridRow


class Bootstrap4GridRowForm(models.ModelForm):
    create = IntegerField(
        label=_('Create columns'),
        help_text=_('Number of columns to create when saving.'),
        required=False,
        min_value=0,
        max_value=GRID_SIZE,
    )

    class Meta:
        model = Bootstrap4GridRow
        fields = '__all__'


class Bootstrap4GridColumnBaseForm(models.ModelForm):
    class Meta:
        model = Bootstrap4GridColumn
        fields = '__all__'


# convert regular text type fields to number
extra_fields_column = {}
for size in DEVICE_SIZES:
    extra_fields_column['{}_col'.format(size)] = IntegerField(
        label='col' if size == 'xs' else 'col-{}'.format(size),
        required=False,
        min_value=1,
        max_value=GRID_SIZE,
    )
    extra_fields_column['{}_order'.format(size)] = IntegerField(
        label='order' if size == 'xs' else 'order-{}'.format(size),
        required=False,
        min_value=0,
        max_value=GRID_SIZE,
    )
    extra_fields_column['{}_offset'.format(size)] = IntegerField(
        label='offset' if size == 'xs' else 'offset-{}'.format(size),
        required=False,
        min_value=0,
        max_value=GRID_SIZE,
    )
    extra_fields_column['{}_ml'.format(size)] = BooleanField(
        label='ml-auto' if size == 'xs' else 'ml-{}-auto'.format(size),
        required=False,
    )
    extra_fields_column['{}_mr'.format(size)] = BooleanField(
        label='mr-auto' if size == 'xs' else 'mr-{}-auto'.format(size),
        required=False,
    )

Bootstrap4GridColumnForm = type(
    str('Bootstrap4GridColumnBaseForm'),
    (Bootstrap4GridColumnBaseForm,),
    extra_fields_column,
)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models
from django.utils.translation import ugettext_lazy as _

from djangocms_picture.models import AbstractPicture
from six import python_2_unicode_compatible


@python_2_unicode_compatible
class Bootstrap4Picture(AbstractPicture):
    """
    Content > "Image" Plugin
    https://getbootstrap.com/docs/4.0/content/images/
    """
    picture_fluid = models.BooleanField(
        verbose_name=_('Responsive'),
        default=True,
        help_text=_('Adds the .img-fluid class to make the image responsive.'),
    )
    picture_rounded = models.BooleanField(
        verbose_name=_('Rounded'),
        default=False,
        help_text=_('Adds the .rounded class for round corners.'),
    )
    picture_thumbnail = models.BooleanField(
        verbose_name=_('Thumbnail'),
        default=False,
        help_text=_('Adds the .img-thumbnail class.'),
    )

    class Meta:
        abstract = False

    def __str__(self):
        return str(self.pk)



# -*- coding: utf-8 -*-
from cms.api import add_plugin
from cms.test_utils.testcases import CMSTestCase

from djangocms_bootstrap4.contrib.bootstrap4_media.cms_plugins import (
    Bootstrap4MediaBodyPlugin, Bootstrap4MediaPlugin,
)

from ..fixtures import B4TestFixture


class B4MediaPluginTestCase(B4TestFixture, CMSTestCase):

    def test_media_plugin(self):
        plugin = add_plugin(
            placeholder=self.placeholder,
            plugin_type=Bootstrap4MediaPlugin.__name__,
            language=self.language,
        )
        plugin.full_clean()
        self.page.publish(self.language)

        with self.login_user_context(self.superuser):
            response = self.client.get(self.request_url)
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, '<div class="media">')

    def test_media_body_plugin(self):
        plugin = add_plugin(
            placeholder=self.placeholder,
            plugin_type=Bootstrap4MediaBodyPlugin.__name__,
            language=self.language,
        )
        plugin.full_clean()
        self.page.publish(self.language)

        with self.login_user_context(self.superuser):
            response = self.client.get(self.request_url)
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, '<div class="media-body">')



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import os

from django.db import models
from django.utils.html import strip_tags
from django.utils.translation import ugettext_lazy as _

from cms.models import CMSPlugin

from djangocms_link.models import AbstractLink
from djangocms_text_ckeditor.fields import HTMLField
from filer.fields.image import FilerImageField
from six import python_2_unicode_compatible

from djangocms_bootstrap4.fields import AttributesField, TagTypeField

from .constants import (
    CAROUSEL_ASPECT_RATIO_CHOICES, CAROUSEL_PAUSE_CHOICES,
    CAROUSEL_RIDE_CHOICES, CAROUSEL_TEMPLATE_CHOICES,
)


@python_2_unicode_compatible
class Bootstrap4Carousel(CMSPlugin):
    """
    Components > "Carousel" Plugin
    https://getbootstrap.com/docs/4.0/components/carousel/
    """
    template = models.CharField(
        verbose_name=_('Template'),
        choices=CAROUSEL_TEMPLATE_CHOICES,
        default=CAROUSEL_TEMPLATE_CHOICES[0][0],
        max_length=255,
        help_text=_('This is the template that will be used for the component.'),
    )
    carousel_interval = models.IntegerField(
        verbose_name=_('Interval'),
        default=5000,
        help_text=_('The amount of time to delay between automatically cycling '
                    'an item. If false, carousel will not automatically cycle.'),
    )
    carousel_controls = models.BooleanField(
        verbose_name=_('Controls'),
        default=True,
        help_text=_('Adding in the previous and next controls.'),
    )
    carousel_indicators = models.BooleanField(
        verbose_name=_('Indicators'),
        default=True,
        help_text=_('Adding in the indicators to the carousel.'),
    )
    carousel_keyboard = models.BooleanField(
        verbose_name=_('Keyboard'),
        default=True,
        help_text=_('Whether the carousel should react to keyboard events.'),
    )
    carousel_pause = models.CharField(
        verbose_name=_('Pause'),
        choices=CAROUSEL_PAUSE_CHOICES,
        default=CAROUSEL_PAUSE_CHOICES[0][0],
        max_length=255,
        help_text=_('If set to "hover", pauses the cycling of the carousel on '
                    '"mouseenter" and resumes the cycling of the carousel on '
                    '"mouseleave". If set to "false", hovering over the carousel '
                    'won\'t pause it.')
    )
    carousel_ride = models.CharField(
        verbose_name=_('Ride'),
        choices=CAROUSEL_RIDE_CHOICES,
        default=CAROUSEL_RIDE_CHOICES[0][0],
        max_length=255,
        help_text=_('Autoplays the carousel after the user manually cycles the '
                    'first item. If "carousel", autoplays the carousel on load.'),
    )
    carousel_wrap = models.BooleanField(
        verbose_name=_('Wrap'),
        default=True,
        help_text=_('Whether the carousel should cycle continuously or have '
                    'hard stops.'),
    )
    carousel_aspect_ratio = models.CharField(
        verbose_name=_('Aspect ratio'),
        choices=CAROUSEL_ASPECT_RATIO_CHOICES,
        blank=True,
        default='',
        max_length=255,
        help_text=_('Determines width and height of the image '
                    'according to the selected ratio.'),
    )
    tag_type = TagTypeField()
    attributes = AttributesField(
        excluded_keys=[
            'id', 'data-interval', 'data-keyboard',
            'data-pause', 'data-ride', 'data-wrap'
        ],
    )

    def __str__(self):
        return str(self.pk)

    def get_short_description(self):
        text = '({})'.format(self.template)
        text += ' {}: {}'.format(_('Interval'), self.carousel_interval)
        text += ', {}: {}'.format(_('Controls'), self.carousel_controls)
        text += ', {}: {}'.format(_('Indicators'), self.carousel_indicators)
        text += ', {}: {}'.format(_('Keyboard'), self.carousel_keyboard)
        text += ', {}: {}'.format(_('Pause'), self.carousel_pause)
        text += ', {}: {}'.format(_('Ride'), self.carousel_ride)
        text += '{}: {}'.format(_('Wrap'), self.carousel_wrap)
        return text


@python_2_unicode_compatible
class Bootstrap4CarouselSlide(AbstractLink):
    """
    Components > "Slide" Plugin
    https://getbootstrap.com/docs/4.0/components/carousel/
    """
    carousel_image = FilerImageField(
        verbose_name=_('Slide image'),
        blank=False,
        null=True,
        on_delete=models.SET_NULL,
        related_name='+',
    )
    carousel_content = HTMLField(
        verbose_name=_('Content'),
        blank=True,
        default='',
        help_text=_('Content may also be added using child plugins.'),
    )
    tag_type = TagTypeField()

    def __str__(self):
        return str(self.pk)

    def clean(self):
        super(AbstractLink, self).clean()

    def get_link(self):
        return AbstractLink.get_link(self)

    def get_short_description(self):
        image_text = content_text = ''

        if self.carousel_image_id:
            if self.carousel_image.name:
                image_text = self.carousel_image.name
            elif self.carousel_image.original_filename \
                    and os.path.split(self.carousel_image.original_filename)[1]:
                image_text = os.path.split(self.carousel_image.original_filename)[1]
            else:
                image_text = 'Image'
        if self.carousel_content:
            text = strip_tags(self.carousel_content).strip()
            if len(text) > 100:
                content_text = '{}...'.format(text[:100])
            else:
                content_text = '{}'.format(text)

        if image_text and content_text:
            return '{} ({})'.format(image_text, content_text)
        else:
            return image_text or content_text



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.utils.translation import ugettext_lazy as _

from cms.plugin_base import CMSPluginBase
from cms.plugin_pool import plugin_pool

from djangocms_bootstrap4.helpers import concat_classes

from .models import Bootstrap4Alerts


class Bootstrap4AlertsPlugin(CMSPluginBase):
    """
    Components > "Alerts" Plugin
    https://getbootstrap.com/docs/4.0/components/alerts/
    """
    model = Bootstrap4Alerts
    name = _('Alert')
    module = _('Bootstrap 4')
    render_template = 'djangocms_bootstrap4/alerts.html'
    change_form_template = 'djangocms_bootstrap4/admin/alerts.html'
    allow_children = True

    fieldsets = [
        (None, {
            'fields': (
                'alert_context',
                'alert_dismissable',
            )
        }),
        (_('Advanced settings'), {
            'classes': ('collapse',),
            'fields': (
                'tag_type',
                'attributes',
            )
        }),
    ]

    def render(self, context, instance, placeholder):
        link_classes = ['alert']
        link_classes.append('alert-{}'.format(instance.alert_context))

        classes = concat_classes(link_classes + [
            instance.attributes.get('class'),
        ])
        instance.attributes['class'] = classes

        return super(Bootstrap4AlertsPlugin, self).render(
            context, instance, placeholder
        )


plugin_pool.register_plugin(Bootstrap4AlertsPlugin)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.utils.translation import ugettext_lazy as _

from cms.plugin_base import CMSPluginBase
from cms.plugin_pool import plugin_pool

from djangocms_link.cms_plugins import LinkPlugin

from djangocms_bootstrap4.helpers import concat_classes, get_plugin_template

from .constants import CAROUSEL_DEFAULT_SIZE, CAROUSEL_TEMPLATE_CHOICES
from .models import Bootstrap4Carousel, Bootstrap4CarouselSlide


class Bootstrap4CarouselPlugin(CMSPluginBase):
    """
    Components > "Carousel" Plugin
    https://getbootstrap.com/docs/4.0/components/carousel/
    """
    model = Bootstrap4Carousel
    name = _('Carousel')
    module = _('Bootstrap 4')
    allow_children = True
    child_classes = ['Bootstrap4CarouselSlidePlugin']

    fieldsets = [
        (None, {
            'fields': (
                ('carousel_aspect_ratio', 'carousel_interval'),
                ('carousel_controls', 'carousel_indicators'),
                ('carousel_keyboard', 'carousel_wrap'),
                ('carousel_ride', 'carousel_pause'),
            )
        }),
        (_('Advanced settings'), {
            'classes': ('collapse',),
            'fields': (
                'template',
                'tag_type',
                'attributes',
            )
        }),
    ]

    def get_render_template(self, context, instance, placeholder):
        return get_plugin_template(
            instance, 'carousel', 'carousel', CAROUSEL_TEMPLATE_CHOICES
        )

    def render(self, context, instance, placeholder):
        link_classes = ['carousel', 'slide']

        classes = concat_classes(link_classes + [
            instance.attributes.get('class'),
        ])
        instance.attributes['class'] = classes

        return super(Bootstrap4CarouselPlugin, self).render(
            context, instance, placeholder
        )


class Bootstrap4CarouselSlidePlugin(LinkPlugin):
    """
    Components > "Carousel Slide" Plugin
    https://getbootstrap.com/docs/4.0/components/carousel/
    """
    model = Bootstrap4CarouselSlide
    name = _('Carousel slide')
    module = _('Bootstrap 4')
    allow_children = True
    parent_classes = ['Bootstrap4CarouselPlugin']

    fieldsets = [
        (None, {
            'fields': (
                'carousel_image',
                'carousel_content',
            )
        }),
        (_('Link settings'), {
            'classes': ('collapse',),
            'fields': (
                ('external_link', 'internal_link'),
                ('mailto', 'phone'),
                ('anchor', 'target'),
            )
        }),
        (_('Advanced settings'), {
            'classes': ('collapse',),
            'fields': (
                'tag_type',
                'attributes',
            )
        }),
    ]

    def render(self, context, instance, placeholder):
        parent = instance.parent.get_plugin_instance()[0]
        width = float(context.get('width') or CAROUSEL_DEFAULT_SIZE[0])
        height = float(context.get('height') or CAROUSEL_DEFAULT_SIZE[1])

        if parent.carousel_aspect_ratio:
            aspect_width, aspect_height = tuple(
                [int(i) for i in parent.carousel_aspect_ratio.split('x')]
            )
            height = width * aspect_height / aspect_width

        link_classes = ['carousel-item']
        if instance.position == 0:
            link_classes.append('active')
        classes = concat_classes(link_classes + [
            instance.attributes.get('class'),
        ])
        instance.attributes['class'] = classes

        context['instance'] = instance
        context['link'] = instance.get_link()
        context['options'] = {
            'crop': 10,
            'size': (width, height),
            'upscale': True
        }
        return context

    def get_render_template(self, context, instance, placeholder):
        return get_plugin_template(
            instance.parent.get_plugin_instance()[0],
            'carousel',
            'slide',
            CAROUSEL_TEMPLATE_CHOICES,
        )


plugin_pool.register_plugin(Bootstrap4CarouselPlugin)
plugin_pool.register_plugin(Bootstrap4CarouselSlidePlugin)



# -*- coding: utf-8 -*-
# Generated by Django 1.9.13 on 2017-10-15 10:02
from __future__ import unicode_literals

import django.db.models.deletion
from django.db import migrations, models

import djangocms_bootstrap4.fields
from djangocms_bootstrap4.constants import TAG_CHOICES

from ..constants import (
    CARD_ALIGNMENT_CHOICES, CARD_INNER_TYPE_CHOICES, CARD_TAG_CHOICES,
    CARD_TYPE_CHOICES,
)
from ..models import CARD_COLOR_STYLE_CHOICES, CARD_TEXT_STYLES


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('cms', '0016_auto_20160608_1535'),
    ]

    operations = [
        migrations.CreateModel(
            name='Bootstrap4Card',
            fields=[
                ('cmsplugin_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, related_name='bootstrap4_card_bootstrap4card', serialize=False, to='cms.CMSPlugin')),
                ('card_type', models.CharField(choices=CARD_TYPE_CHOICES, default=CARD_TYPE_CHOICES[0][0], max_length=255, verbose_name='Card type')),
                ('card_context', models.CharField(blank=True, choices=CARD_COLOR_STYLE_CHOICES, max_length=255, verbose_name='Background context')),
                ('card_alignment', models.CharField(blank=True, choices=CARD_ALIGNMENT_CHOICES, max_length=255, verbose_name='Alignment')),
                ('card_outline', models.BooleanField(default=False, help_text='Uses the border context instead of the background.', verbose_name='Outline')),
                ('card_text_color', models.CharField(blank=True, choices=CARD_TEXT_STYLES, max_length=255, verbose_name='Text context')),
                ('tag_type', djangocms_bootstrap4.fields.TagTypeField(choices=TAG_CHOICES, default=TAG_CHOICES[0][0], help_text='Select the HTML tag to be used.', max_length=255, verbose_name='Tag type')),
                ('attributes', djangocms_bootstrap4.fields.AttributesField(blank=True, default=dict, verbose_name='Attributes')),
            ],
            options={
                'abstract': False,
            },
            bases=('cms.cmsplugin',),
        ),
        migrations.CreateModel(
            name='Bootstrap4CardInner',
            fields=[
                ('cmsplugin_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, related_name='bootstrap4_card_bootstrap4cardinner', serialize=False, to='cms.CMSPlugin')),
                ('inner_type', models.CharField(choices=CARD_INNER_TYPE_CHOICES, default=CARD_INNER_TYPE_CHOICES[0][0], help_text='Define the structure of the plugin.', max_length=255, verbose_name='Inner type')),
                ('tag_type', djangocms_bootstrap4.fields.TagTypeField(choices=TAG_CHOICES, default=TAG_CHOICES[0][0], help_text='Select the HTML tag to be used.', max_length=255, verbose_name='Tag type')),
                ('attributes', djangocms_bootstrap4.fields.AttributesField(blank=True, default=dict, verbose_name='Attributes')),
            ],
            options={
                'abstract': False,
            },
            bases=('cms.cmsplugin',),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.conf import settings
from django.utils.translation import ugettext_lazy as _


CAROUSEL_PAUSE_CHOICES = (
    ('hover', 'hover'),
    ('mouseenter', 'mouseenter'),
    ('mouseleave', 'mouseleave'),
    ('false', 'off'),
)

CAROUSEL_RIDE_CHOICES = (
    ('carousel', 'carousel'),
    ('false', 'off'),
)

CAROUSEL_TEMPLATE_CHOICES = getattr(
    settings,
    'DJANGOCMS_BOOTSTRAP4_CAROUSEL_TEMPLATES',
    (
        ('default', _('Default')),
    ),
)

# this is used when no valua is passed in the template via
# {% with 1024 as width and 768 as height %}
CAROUSEL_DEFAULT_SIZE = getattr(
    settings,
    'DJANGOCMS_BOOTSTRAP4_CAROUSEL_DEFAULT_SIZE',
    [1024, 768],
)

CAROUSEL_ASPECT_RATIOS = (
    (1, 1),
    (3, 2),
    (4, 3),
    (21, 9),
    (18, 9),
) + tuple(
    getattr(
        settings,
        'DJANGOCMS_BOOTSTRAP4_CAROUSEL_ASPECT_RATIOS',
        tuple()
    )
)

CAROUSEL_ASPECT_RATIO_CHOICES = (
    tuple([
        ('{0}x{1}'.format(x, y), '{0}x{1}'.format(x, y))
        for x, y in CAROUSEL_ASPECT_RATIOS
    ])
)



# -*- coding: utf-8 -*-
import warnings

from cms.api import add_plugin
from cms.test_utils.testcases import CMSTestCase

from djangocms_bootstrap4.contrib.bootstrap4_grid.cms_plugins import (
    Bootstrap4GridColumnPlugin, Bootstrap4GridContainerPlugin,
    Bootstrap4GridRowPlugin,
)

from ..fixtures import B4TestFixture


class B4GridPluginTestCase(B4TestFixture, CMSTestCase):

    def test_container_plugin(self):
        plugin = add_plugin(
            placeholder=self.placeholder,
            plugin_type=Bootstrap4GridContainerPlugin.__name__,
            language=self.language,
        )
        plugin.full_clean()
        self.assertEqual(
            plugin.plugin_type,
            "Bootstrap4GridContainerPlugin",
        )

    def test_grid_row_plugin(self):
        plugin = add_plugin(
            placeholder=self.placeholder,
            plugin_type=Bootstrap4GridRowPlugin.__name__,
            language=self.language,
        )
        plugin.full_clean()
        self.assertEqual(
            plugin.plugin_type,
            "Bootstrap4GridRowPlugin",
        )

    def test_grid_column_plugin(self):
        plugin = add_plugin(
            placeholder=self.placeholder,
            plugin_type=Bootstrap4GridColumnPlugin.__name__,
            language=self.language,
        )
        plugin.full_clean()
        self.assertEqual(
            plugin.plugin_type,
            "Bootstrap4GridColumnPlugin",
        )

    def test_plugin_structure(self):
        container = add_plugin(
            placeholder=self.placeholder,
            plugin_type=Bootstrap4GridContainerPlugin.__name__,
            language=self.language,
        )
        self.page.publish(self.language)

        with self.login_user_context(self.superuser):
            response = self.client.get(self.request_url)
        self.assertEqual(response.status_code, 200)

        row = add_plugin(
            target=container,
            placeholder=self.placeholder,
            plugin_type=Bootstrap4GridRowPlugin.__name__,
            language=self.language,
        )
        self.page.publish(self.language)
        with self.login_user_context(self.superuser):
            response = self.client.get(self.request_url)
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, '<div class="row">')

        # add column with values
        add_plugin(
            target=row,
            placeholder=self.placeholder,
            plugin_type=Bootstrap4GridColumnPlugin.__name__,
            language=self.language,
            xs_col=12,
        )
        self.page.publish(self.language)
        with self.login_user_context(self.superuser):
            response = self.client.get(self.request_url)
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, '<div class="col col-12">')

        # add row without values
        add_plugin(
            target=row,
            placeholder=self.placeholder,
            plugin_type=Bootstrap4GridColumnPlugin.__name__,
            language=self.language,
        )
        self.page.publish(self.language)
        with self.login_user_context(self.superuser):
            response = self.client.get(self.request_url)
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, '<div class="col">')

    def test_row_plugin_creation(self):
        request_url = self.get_add_plugin_uri(
            placeholder=self.placeholder,
            plugin_type=Bootstrap4GridRowPlugin.__name__,
            language=self.language,
        )
        # create 5 column plugins
        data = {
            "create": 5,
            "tag_type": "div"
        }

        with self.login_user_context(self.superuser), warnings.catch_warnings():
            # hide the "DontUsePageAttributeWarning" warning when using
            # `get_add_plugin_uri` to get cleaner test results
            warnings.simplefilter("ignore")
            response = self.client.post(request_url, data)
        self.assertEquals(response.status_code, 200)
        self.assertContains(response, '<div class="success">')



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models
from django.utils.translation import ugettext_lazy as _

from cms.models import CMSPlugin

from six import python_2_unicode_compatible

from djangocms_bootstrap4.constants import COLOR_STYLE_CHOICES
from djangocms_bootstrap4.fields import AttributesField


@python_2_unicode_compatible
class Bootstrap4Badge(CMSPlugin):
    """
    Components > "Badge" Plugin
    https://getbootstrap.com/docs/4.0/components/badge/
    """
    badge_text = models.CharField(
        verbose_name=_('Badge text'),
        max_length=255,
    )
    badge_context = models.CharField(
        verbose_name=_('Context'),
        choices=COLOR_STYLE_CHOICES,
        default=COLOR_STYLE_CHOICES[0][0],
        max_length=255,
    )
    badge_pills = models.BooleanField(
        verbose_name=_('Pills style'),
        default=False,
        help_text=_('Activates the pills style.'),
    )

    attributes = AttributesField()

    def __str__(self):
        return str(self.pk)

    def get_short_description(self):
        return '({})'.format(self.badge_context)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.utils.translation import ugettext_lazy as _

from cms.plugin_pool import plugin_pool

from djangocms_link.cms_plugins import LinkPlugin
from djangocms_link.models import get_templates

from djangocms_bootstrap4.helpers import concat_classes, get_plugin_template

from .constants import USE_LINK_ICONS
from .forms import Bootstrap4LinkForm
from .models import Bootstrap4Link


class Bootstrap4LinkPlugin(LinkPlugin):
    """
    Components > "Button" Plugin
    https://getbootstrap.com/docs/4.0/components/buttons/
    """
    model = Bootstrap4Link
    name = _('Link / Button')
    form = Bootstrap4LinkForm
    change_form_template = 'djangocms_bootstrap4/admin/link.html'
    module = _('Bootstrap 4')

    fields = (
        ('name', 'link_type'),
        ('external_link', 'internal_link'),
        ('link_context', 'link_size'),
        ('link_outline', 'link_block'),
    )

    if USE_LINK_ICONS:  # pragma: no cover
        fields = fields + (
            ('icon_left', 'icon_right'),
        )

    LinkPlugin.fieldsets[0] = (
        None, {
            'fields': fields
        }
    )

    fieldsets = LinkPlugin.fieldsets

    def get_render_template(self, context, instance, placeholder):
        return get_plugin_template(
            instance, 'link', 'link', get_templates()
        )

    def render(self, context, instance, placeholder):
        link_classes = []
        if instance.link_context:
            if instance.link_type == 'link':
                link_classes.append('text-{}'.format(instance.link_context))
            else:
                link_classes.append('btn')
                if not instance.link_outline:
                    link_classes.append(
                        'btn-{}'.format(instance.link_context)
                    )
                else:
                    link_classes.append(
                        'btn-outline-{}'.format(instance.link_context)
                    )
        if instance.link_size:
            link_classes.append(instance.link_size)
        if instance.link_block:
            link_classes.append('btn-block')

        classes = concat_classes(link_classes + [
            instance.attributes.get('class'),
        ])
        instance.attributes['class'] = classes

        return super(Bootstrap4LinkPlugin, self).render(
            context, instance, placeholder
        )


plugin_pool.unregister_plugin(LinkPlugin)
plugin_pool.register_plugin(Bootstrap4LinkPlugin)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models
from django.utils.translation import ugettext_lazy as _

from cms.models import CMSPlugin

from six import python_2_unicode_compatible

from djangocms_bootstrap4.constants import COLOR_STYLE_CHOICES
from djangocms_bootstrap4.fields import AttributesField, TagTypeField

from .constants import (
    CARD_ALIGNMENT_CHOICES, CARD_INNER_TYPE_CHOICES, CARD_TYPE_CHOICES,
)


# cards allow for a transparent color
CARD_COLOR_STYLE_CHOICES = COLOR_STYLE_CHOICES + (
    ('transparent', _('Transparent')),
)

CARD_TEXT_STYLES = COLOR_STYLE_CHOICES + (
    ('white', _('White')),
)


@python_2_unicode_compatible
class Bootstrap4Card(CMSPlugin):
    """
    Components > "Card" Plugin
    https://getbootstrap.com/docs/4.0/components/card/
    """
    card_type = models.CharField(
        verbose_name=_('Card type'),
        choices=CARD_TYPE_CHOICES,
        default=CARD_TYPE_CHOICES[0][0],
        max_length=255,
    )
    card_context = models.CharField(
        verbose_name=_('Background context'),
        choices=CARD_COLOR_STYLE_CHOICES,
        blank=True,
        max_length=255,
    )
    card_alignment = models.CharField(
        verbose_name=_('Alignment'),
        choices=CARD_ALIGNMENT_CHOICES,
        blank=True,
        max_length=255,
    )
    card_outline = models.BooleanField(
        verbose_name=_('Outline'),
        default=False,
        help_text=_('Uses the border context instead of the background.'),
    )
    card_text_color = models.CharField(
        verbose_name=_('Text context'),
        choices=CARD_TEXT_STYLES,
        blank=True,
        max_length=255,
    )
    tag_type = TagTypeField()
    attributes = AttributesField()

    def __str__(self):
        return str(self.pk)

    def get_short_description(self):
        text = '({})'.format(self.card_type)
        if self.card_context and self.card_outline:
            text += ' .border-{}'.format(self.card_context)
        elif self.card_context:
            text += ' .bg-{}'.format(self.card_context)
        if self.card_alignment:
            text += ' .{}'.format(self.card_alignment)
        return text


@python_2_unicode_compatible
class Bootstrap4CardInner(CMSPlugin):
    """
    Components > "Card - Inner" Plugin (Header, Footer, Body)
    https://getbootstrap.com/docs/4.0/components/card/
    """
    inner_type = models.CharField(
        verbose_name=_('Inner type'),
        choices=CARD_INNER_TYPE_CHOICES,
        default=CARD_INNER_TYPE_CHOICES[0][0],
        max_length=255,
        help_text=_('Define the structure of the plugin.'),
    )
    tag_type = TagTypeField()
    attributes = AttributesField()

    def __str__(self):
        return str(self.pk)

    def get_short_description(self):
        return '({})'.format(self.inner_type)



# -*- coding: utf-8 -*-
from django.test import TestCase

from djangocms_bootstrap4.contrib.bootstrap4_content.models import (
    Bootstrap4Blockquote, Bootstrap4Code, Bootstrap4Figure,
)


class B4ContentModelTestCase(TestCase):

    def test_code_instance(self):
        instance = Bootstrap4Code.objects.create()
        self.assertEqual(str(instance), "1")
        self.assertEqual(instance.get_short_description(), "<code>")

    def test_blockquote_instance(self):
        instance = Bootstrap4Blockquote.objects.create()
        self.assertEqual(str(instance), "1")
        self.assertEqual(instance.get_short_description(), "")

    def test_figure_instance(self):
        instance = Bootstrap4Figure.objects.create()
        self.assertEqual(str(instance), "1")
        self.assertEqual(instance.get_short_description(), "")



# -*- coding: utf-8 -*-
from cms.api import add_plugin
from cms.test_utils.testcases import CMSTestCase

from djangocms_bootstrap4.contrib.bootstrap4_collapse.cms_plugins import (
    Bootstrap4CollapseContainerPlugin, Bootstrap4CollapsePlugin,
    Bootstrap4CollapseTriggerPlugin,
)

from ..fixtures import B4TestFixture


class B4CollapsePluginTestCase(B4TestFixture, CMSTestCase):

    def test_collapse_plugin(self):
        plugin = add_plugin(
            placeholder=self.placeholder,
            plugin_type=Bootstrap4CollapsePlugin.__name__,
            language=self.language,
        )
        plugin.full_clean()
        self.page.publish(self.language)

        with self.login_user_context(self.superuser):
            response = self.client.get(self.request_url)
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, 'data-children=".card"')
        self.assertContains(response, 'role="tablist"')

    def test_collapse_trigger_plugin(self):
        plugin = add_plugin(
            placeholder=self.placeholder,
            plugin_type=Bootstrap4CollapseTriggerPlugin.__name__,
            language=self.language,
            identifier=10,
        )
        plugin.full_clean()
        self.page.publish(self.language)

        with self.login_user_context(self.superuser):
            response = self.client.get(self.request_url)
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, 'aria-controls="container-10"')
        self.assertContains(response, 'data-target="#container-10"')
        self.assertContains(response, 'id="trigger-10"')

    def test_collapse_container_plugin(self):
        parent = add_plugin(
            placeholder=self.placeholder,
            plugin_type=Bootstrap4CollapsePlugin.__name__,
            language=self.language,
        )
        plugin = add_plugin(
            target=parent,
            placeholder=self.placeholder,
            plugin_type=Bootstrap4CollapseContainerPlugin.__name__,
            language=self.language,
            identifier=10,
        )
        plugin.full_clean()
        self.page.publish(self.language)

        with self.login_user_context(self.superuser):
            response = self.client.get(self.request_url)
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, 'aria-labelledby="trigger-10"')
        self.assertContains(response, 'container-10')



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django import forms

from djangocms_icon.fields import IconField
from djangocms_link.forms import LinkForm

from .constants import LINK_CHOICES
from .models import Bootstrap4Link


class Bootstrap4LinkForm(LinkForm):
    link_type = forms.ChoiceField(
        choices=LINK_CHOICES,
        initial=LINK_CHOICES[0][0],
        widget=forms.RadioSelect(attrs={'class': 'inline-block'}),
    )
    icon_left = IconField(required=False)
    icon_right = IconField(required=False)

    class Meta:
        model = Bootstrap4Link
        fields = '__all__'



# -*- coding: utf-8 -*-
# Generated by Django 1.9.13 on 2017-10-15 12:54
from __future__ import unicode_literals

import django.db.models.deletion
from django.db import migrations, models

import djangocms_bootstrap4.fields
from djangocms_bootstrap4.constants import COLOR_STYLE_CHOICES


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('cms', '0016_auto_20160608_1535'),
    ]

    operations = [
        migrations.CreateModel(
            name='Bootstrap4Badge',
            fields=[
                ('cmsplugin_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, related_name='bootstrap4_badge_bootstrap4badge', serialize=False, to='cms.CMSPlugin')),
                ('badge_text', models.CharField(max_length=255, verbose_name='Badge text')),
                ('badge_context', models.CharField(choices=COLOR_STYLE_CHOICES, default=COLOR_STYLE_CHOICES[0][0], max_length=255, verbose_name='Context')),
                ('badge_pills', models.BooleanField(default=False, help_text='Activates the pills style.', verbose_name='Pills style')),
                ('attributes', djangocms_bootstrap4.fields.AttributesField(blank=True, default=dict, verbose_name='Attributes')),
            ],
            options={
                'abstract': False,
            },
            bases=('cms.cmsplugin',),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('bootstrap4_grid', '0003_migrate_column_size'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='bootstrap4gridcolumn',
            name='column_size',
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.13 on 2017-10-12 06:33
from __future__ import unicode_literals

import django.db.models.deletion
from django.db import migrations, models

import djangocms_attributes_field.fields
import djangocms_link.validators
from djangocms_link.models import HOSTNAME, TARGET_CHOICES, get_templates

from ..constants import LINK_CHOICES, LINK_SIZE_CHOICES
from ..models import COLOR_STYLE_CHOICES


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('cms', '0016_auto_20160608_1535'),
    ]

    operations = [
        migrations.CreateModel(
            name='Bootstrap4Link',
            fields=[
                ('template', models.CharField(choices=get_templates(), default=get_templates()[0][0], max_length=255, verbose_name='Template')),
                ('name', models.CharField(blank=True, max_length=255, verbose_name='Display name')),
                ('external_link', models.URLField(blank=True, help_text='Provide a valid URL to an external website.', max_length=2040, validators=[djangocms_link.validators.IntranetURLValidator(intranet_host_re=HOSTNAME)], verbose_name='External link')),
                ('anchor', models.CharField(blank=True, help_text='Appends the value only after the internal or external link. Do <em>not</em> include a preceding "#" symbol.', max_length=255, verbose_name='Anchor')),
                ('mailto', models.EmailField(blank=True, max_length=255, verbose_name='Email address')),
                ('phone', models.CharField(blank=True, max_length=255, verbose_name='Phone')),
                ('target', models.CharField(blank=True, choices=TARGET_CHOICES, max_length=255, verbose_name='Target')),
                ('attributes', djangocms_attributes_field.fields.AttributesField(blank=True, default=dict, verbose_name='Attributes')),
                ('cmsplugin_ptr', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, related_name='bootstrap4_link_bootstrap4link', serialize=False, to='cms.CMSPlugin')),
                ('link_type', models.CharField(choices=LINK_CHOICES, default=LINK_CHOICES[0][0], help_text='Adds either the .btn-* or .text-* classes.', max_length=255, verbose_name='Type')),
                ('link_context', models.CharField(blank=True, choices=COLOR_STYLE_CHOICES, max_length=255, verbose_name='Context')),
                ('link_size', models.CharField(blank=True, choices=LINK_SIZE_CHOICES, max_length=255, verbose_name='Size')),
                ('link_outline', models.BooleanField(default=False, help_text='Applies the .btn-outline class to the elements.', verbose_name='Outline')),
                ('link_block', models.BooleanField(default=False, help_text='Extends the button to the width of its container.', verbose_name='Block')),
                ('internal_link', models.ForeignKey(blank=True, help_text='If provided, overrides the external link.', null=True, on_delete=django.db.models.deletion.SET_NULL, to='cms.Page', verbose_name='Internal link')),
            ],
            options={
                'abstract': False,
            },
            bases=('cms.cmsplugin',),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models
from django.utils.translation import ugettext_lazy as _

from cms.models import CMSPlugin

from six import python_2_unicode_compatible

from djangocms_bootstrap4.constants import DEVICE_CHOICES
from djangocms_bootstrap4.fields import AttributesField, TagTypeField

from .constants import (
    SPACER_PROPERTY_CHOICES, SPACER_SIDE_CHOICES, SPACER_SIZE_CHOICES,
)


@python_2_unicode_compatible
class Bootstrap4Spacing(CMSPlugin):
    """
    Utilities > "Spacing" Plugin
    https://getbootstrap.com/docs/4.0/utilities/spacing/
    """
    space_property = models.CharField(
        verbose_name=_('Property'),
        choices=SPACER_PROPERTY_CHOICES,
        default=SPACER_PROPERTY_CHOICES[0][0],
        max_length=255,
    )
    space_sides = models.CharField(
        verbose_name=_('Sides'),
        choices=SPACER_SIDE_CHOICES,
        default=SPACER_SIDE_CHOICES[0][0],
        blank=True,
        max_length=255,
    )
    space_size = models.CharField(
        verbose_name=_('Size'),
        choices=SPACER_SIZE_CHOICES,
        default=SPACER_SIZE_CHOICES[0][0],
        max_length=255,
    )
    space_device = models.CharField(
        verbose_name=_('Device'),
        choices=DEVICE_CHOICES,
        blank=True,
        max_length=255,
    )
    tag_type = TagTypeField()
    attributes = AttributesField()

    def __str__(self):
        return str(self.pk)

    def get_base_css_class(self):
        # Source: https://getbootstrap.com/docs/4.0/utilities/spacing/#notation
        # [...] format {property}{sides}-{size} for xs and
        # {property}{sides}-{breakpoint}-{size} for sm, md, lg, and xl.
        if not self.space_device or self.space_device == 'xs':
            template = '{property}{sides}-{size}'
        else:
            template = '{property}{sides}-{breakpoint}-{size}'

        return template.format(
            property=self.space_property,
            sides=self.space_sides,
            breakpoint=self.space_device,
            size=self.space_size
        )

    def get_short_description(self):
        return '(.{})'.format(self.get_base_css_class())



# -*- coding: utf-8 -*-
from cms.api import add_plugin
from cms.test_utils.testcases import CMSTestCase

from djangocms_bootstrap4.contrib.bootstrap4_link.cms_plugins import (
    Bootstrap4LinkPlugin,
)

from ..fixtures import B4TestFixture


class B4LinkPluginTestCase(B4TestFixture, CMSTestCase):

    def test_plugin(self):
        plugin = add_plugin(
            placeholder=self.placeholder,
            plugin_type=Bootstrap4LinkPlugin.__name__,
            language=self.language,
            external_link="https://www.divio.com",
        )
        plugin.full_clean()
        self.page.publish(self.language)

        with self.login_user_context(self.superuser):
            response = self.client.get(self.request_url)
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, 'href="https://www.divio.com"')

        # add more options
        plugin = add_plugin(
            placeholder=self.placeholder,
            plugin_type=Bootstrap4LinkPlugin.__name__,
            language=self.language,
            external_link="https://www.divio.com",
            link_context="primary",
            link_size="btn-sm",
            link_block=True,
        )
        plugin.full_clean()
        self.page.publish(self.language)

        with self.login_user_context(self.superuser):
            response = self.client.get(self.request_url)
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, 'class="text-primary btn-sm btn-block"')

        # alternate version for link_type
        plugin = add_plugin(
            placeholder=self.placeholder,
            plugin_type=Bootstrap4LinkPlugin.__name__,
            language=self.language,
            external_link="https://www.divio.com",
            link_context="primary",
            link_type="btn",
        )
        plugin.full_clean()
        self.page.publish(self.language)

        with self.login_user_context(self.superuser):
            response = self.client.get(self.request_url)
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, 'class="btn btn-primary"')

        # alternate version using link_outline
        plugin = add_plugin(
            placeholder=self.placeholder,
            plugin_type=Bootstrap4LinkPlugin.__name__,
            language=self.language,
            external_link="https://www.divio.com",
            link_context="primary",
            link_type="btn",
            link_outline=True,
        )
        plugin.full_clean()
        self.page.publish(self.language)

        with self.login_user_context(self.superuser):
            response = self.client.get(self.request_url)
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, 'class="btn btn-outline-primary"')



# -*- coding: utf-8 -*-
# Generated by Django 1.9.13 on 2017-09-27 21:19
from __future__ import unicode_literals

import django.db.models.deletion
from django.db import migrations, models

import djangocms_bootstrap4.fields
from djangocms_bootstrap4.constants import TAG_CHOICES

from ..constants import (
    GRID_COLUMN_ALIGNMENT_CHOICES, GRID_COLUMN_CHOICES, GRID_CONTAINER_CHOICES,
    GRID_ROW_HORIZONTAL_ALIGNMENT_CHOICES, GRID_ROW_VERTICAL_ALIGNMENT_CHOICES,
)


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('cms', '0016_auto_20160608_1535'),
    ]

    operations = [
        migrations.CreateModel(
            name='Bootstrap4GridColumn',
            fields=[
                ('cmsplugin_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, related_name='bootstrap4_grid_bootstrap4gridcolumn', serialize=False, to='cms.CMSPlugin')),
                ('column_type', models.CharField(blank=True, choices=GRID_COLUMN_CHOICES, default=GRID_COLUMN_CHOICES[0][0], max_length=255, verbose_name='Column type')),
                ('column_size', djangocms_bootstrap4.fields.IntegerRangeField(blank=True, help_text='Nummeric value from 1 - 12. Spreads the columns evenly when empty.', null=True, verbose_name='Column size')),
                ('column_alignment', models.CharField(blank=True, choices=GRID_COLUMN_ALIGNMENT_CHOICES, max_length=255, verbose_name='Alignment')),
                ('tag_type', djangocms_bootstrap4.fields.TagTypeField(choices=TAG_CHOICES, default=TAG_CHOICES[0][0], help_text='Select the HTML tag to be used.', max_length=255, verbose_name='Tag type')),
                ('attributes', djangocms_bootstrap4.fields.AttributesField(blank=True, default=dict, verbose_name='Attributes')),
                ('xs_col', djangocms_bootstrap4.fields.IntegerRangeField(blank=True, null=True)),
                ('xs_order', djangocms_bootstrap4.fields.IntegerRangeField(blank=True, null=True)),
                ('xs_ml', models.BooleanField(default=False)),
                ('xs_mr', models.BooleanField(default=False)),
                ('sm_col', djangocms_bootstrap4.fields.IntegerRangeField(blank=True, null=True)),
                ('sm_order', djangocms_bootstrap4.fields.IntegerRangeField(blank=True, null=True)),
                ('sm_ml', models.BooleanField(default=False)),
                ('sm_mr', models.BooleanField(default=False)),
                ('md_col', djangocms_bootstrap4.fields.IntegerRangeField(blank=True, null=True)),
                ('md_order', djangocms_bootstrap4.fields.IntegerRangeField(blank=True, null=True)),
                ('md_ml', models.BooleanField(default=False)),
                ('md_mr', models.BooleanField(default=False)),
                ('lg_col', djangocms_bootstrap4.fields.IntegerRangeField(blank=True, null=True)),
                ('lg_order', djangocms_bootstrap4.fields.IntegerRangeField(blank=True, null=True)),
                ('lg_ml', models.BooleanField(default=False)),
                ('lg_mr', models.BooleanField(default=False)),
                ('xl_col', djangocms_bootstrap4.fields.IntegerRangeField(blank=True, null=True)),
                ('xl_order', djangocms_bootstrap4.fields.IntegerRangeField(blank=True, null=True)),
                ('xl_ml', models.BooleanField(default=False)),
                ('xl_mr', models.BooleanField(default=False)),
            ],
            options={
                'abstract': False,
            },
            bases=('cms.cmsplugin',),
        ),
        migrations.CreateModel(
            name='Bootstrap4GridContainer',
            fields=[
                ('cmsplugin_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, related_name='bootstrap4_grid_bootstrap4gridcontainer', serialize=False, to='cms.CMSPlugin')),
                ('container_type', models.CharField(choices=GRID_CONTAINER_CHOICES, default=GRID_CONTAINER_CHOICES[0][0], help_text='Defines if the grid should use fixed width (<code>.container</code>) or fluid width (<code>.container-fluid</code>).', max_length=255, verbose_name='Container type')),
                ('tag_type', djangocms_bootstrap4.fields.TagTypeField(choices=TAG_CHOICES, default=TAG_CHOICES[0][0], help_text='Select the HTML tag to be used.', max_length=255, verbose_name='Tag type')),
                ('attributes', djangocms_bootstrap4.fields.AttributesField(blank=True, default=dict, verbose_name='Attributes')),
            ],
            options={
                'abstract': False,
            },
            bases=('cms.cmsplugin',),
        ),
        migrations.CreateModel(
            name='Bootstrap4GridRow',
            fields=[
                ('cmsplugin_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, related_name='bootstrap4_grid_bootstrap4gridrow', serialize=False, to='cms.CMSPlugin')),
                ('vertical_alignment', models.CharField(blank=True, choices=GRID_ROW_VERTICAL_ALIGNMENT_CHOICES, help_text='Read more in the <a href="https://getbootstrap.com/docs/4.0/layout/grid/#vertical-alignment" target="_blank">documentation</a>.', max_length=255, verbose_name='Vertical alignment')),
                ('horizontal_alignment', models.CharField(blank=True, choices=GRID_ROW_HORIZONTAL_ALIGNMENT_CHOICES, help_text='Read more in the <a href="https://getbootstrap.com/docs/4.0/layout/grid/#horizontal-alignment" target="_blank">documentation</a>.', max_length=255, verbose_name='Horizontal alignment')),
                ('gutters', models.BooleanField(default=False, help_text='Removes the marginal gutters from the grid.', verbose_name='Remove gutters')),
                ('tag_type', djangocms_bootstrap4.fields.TagTypeField(choices=TAG_CHOICES, default=TAG_CHOICES[0][0], help_text='Select the HTML tag to be used.', max_length=255, verbose_name='Tag type')),
                ('attributes', djangocms_bootstrap4.fields.AttributesField(blank=True, default=dict, verbose_name='Attributes')),
            ],
            options={
                'abstract': False,
            },
            bases=('cms.cmsplugin',),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.utils.translation import ugettext_lazy as _

from cms.plugin_base import CMSPluginBase
from cms.plugin_pool import plugin_pool

from djangocms_bootstrap4.helpers import concat_classes

from .models import Bootstrap4Card, Bootstrap4CardInner


class Bootstrap4CardPlugin(CMSPluginBase):
    """
    Components > "Card" Plugin
    https://getbootstrap.com/docs/4.0/components/card/
    """
    model = Bootstrap4Card
    name = _('Card')
    module = _('Bootstrap 4')
    render_template = 'djangocms_bootstrap4/card.html'
    change_form_template = 'djangocms_bootstrap4/admin/card.html'
    allow_children = True
    child_classes = [
        'Bootstrap4CardPlugin',
        'Bootstrap4CardInnerPlugin',
        'Bootstrap4LinkPlugin',
        'Bootstrap4ListGroupPlugin',
        'Bootstrap4PicturePlugin',
    ]

    fieldsets = [
        (None, {
            'fields': (
                'card_type',
                ('card_context', 'card_text_color'),
                ('card_alignment', 'card_outline'),
            )
        }),
        (_('Advanced settings'), {
            'classes': ('collapse',),
            'fields': (
                'tag_type',
                'attributes',
            )
        }),
    ]

    def render(self, context, instance, placeholder):
        link_classes = [instance.card_type]
        if instance.card_context and instance.card_outline:
            link_classes.append('border-{}'.format(instance.card_context))
        elif instance.card_context:
            link_classes.append('bg-{}'.format(instance.card_context))
        if instance.card_alignment:
            link_classes.append(instance.card_alignment)
        if instance.card_text_color:
            link_classes.append('text-{}'.format(instance.card_text_color))

        classes = concat_classes(link_classes + [
            instance.attributes.get('class'),
        ])
        instance.attributes['class'] = classes

        return super(Bootstrap4CardPlugin, self).render(
            context, instance, placeholder
        )


class Bootstrap4CardInnerPlugin(CMSPluginBase):
    """
    Components > "Card - Inner" Plugin (Header, Footer, Body)
    https://getbootstrap.com/docs/4.0/components/card/
    """
    model = Bootstrap4CardInner
    name = _('Card inner')
    module = _('Bootstrap 4')
    render_template = 'djangocms_bootstrap4/card.html'
    change_form_template = 'djangocms_bootstrap4/admin/card.html'
    allow_children = True
    parent_classes = [
        'Bootstrap4CardPlugin',
        'Bootstrap4CollapseTriggerPlugin',
        'Bootstrap4CollapseContainerPlugin',
    ]

    fieldsets = [
        (None, {
            'fields': (
                'inner_type',
            )
        }),
        (_('Advanced settings'), {
            'classes': ('collapse',),
            'fields': (
                'tag_type',
                'attributes',
            )
        }),
    ]

    def render(self, context, instance, placeholder):
        classes = concat_classes([instance.inner_type] + [
            instance.attributes.get('class'),
        ])
        instance.attributes['class'] = classes

        return super(Bootstrap4CardInnerPlugin, self).render(
            context, instance, placeholder
        )


plugin_pool.register_plugin(Bootstrap4CardPlugin)
plugin_pool.register_plugin(Bootstrap4CardInnerPlugin)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models
from django.utils.translation import ugettext_lazy as _

from cms.models import CMSPlugin

from six import python_2_unicode_compatible

from djangocms_bootstrap4.fields import AttributesField, TagTypeField


# TODO leaving this comment for now
# data-toggle="collapse" data-target="#collapseExample"
# aria-expanded="false" aria-controls="collapseExample">
# data-target can also be classes
# data-parent links to the wrapper collapse
# <div class="collapse" id="collapseExample">


@python_2_unicode_compatible
class Bootstrap4Collapse(CMSPlugin):
    """
    Component > "Collapse" Plugin
    https://getbootstrap.com/docs/4.0/components/collapse/
    """
    siblings = models.CharField(
        verbose_name=_('Siblings'),
        default='.card',
        max_length=255,
        help_text=_('Element to be used to create accordions.'),
    )
    tag_type = TagTypeField()
    attributes = AttributesField(
        excluded_keys=['id']
    )

    def __str__(self):
        return str(self.pk)

    def get_short_description(self):
        return '(collapse-{})'.format(str(self.pk))


@python_2_unicode_compatible
class Bootstrap4CollapseTrigger(CMSPlugin):
    """
    Component > "Collapse Trigger" Plugin
    https://getbootstrap.com/docs/4.0/components/collapse/
    """
    identifier = models.SlugField(
        verbose_name=_('Unique identifier'),
        max_length=255,
        blank=False,
        help_text=_('Identifier to connect trigger with container.'),
    )
    tag_type = TagTypeField()
    attributes = AttributesField(
        excluded_keys=[
            'data-toggle', 'data-target', 'data-parent',
            'aria-expanded', 'aria-controls', 'role',
        ]
    )

    def __str__(self):
        return str(self.pk)

    def get_short_description(self):
        return '({})'.format(self.identifier)


@python_2_unicode_compatible
class Bootstrap4CollapseContainer(CMSPlugin):
    """
    Component > "Collapse Container" Plugin
    https://getbootstrap.com/docs/4.0/components/collapse/
    """
    identifier = models.SlugField(
        verbose_name=_('Unique identifier'),
        max_length=255,
        blank=False,
        help_text=_('Identifier to connect trigger with container.'),
    )
    tag_type = TagTypeField()
    attributes = AttributesField(
        excluded_keys=[
            'data-toggle', 'data-target', 'data-parent',
            'aria-expanded', 'aria-controls', 'role',
        ]
    )

    def __str__(self):
        return str(self.pk)

    def get_short_description(self):
        return '({})'.format(self.identifier)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.utils.translation import ugettext_lazy as _


CODE_TYPE_CHOICES = (
    ('code', _('Inline code')),
    ('pre', _('Code block')),
    ('var', _('Variables')),
    ('kbd', _('User input')),
    ('samp', _('Sample output')),
)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models

import djangocms_bootstrap4.fields
from djangocms_bootstrap4.constants import TAG_CHOICES

from ..constants import (
    TAB_ALIGNMENT_CHOICES, TAB_EFFECT_CHOICES, TAB_TEMPLATE_CHOICES,
    TAB_TYPE_CHOICES,
)


class Migration(migrations.Migration):

    dependencies = [
        ('cms', '0016_auto_20160608_1535'),
    ]

    operations = [
        migrations.CreateModel(
            name='Bootstrap4Tab',
            fields=[
                ('cmsplugin_ptr', models.OneToOneField(parent_link=True, serialize=False, primary_key=True, to='cms.CMSPlugin', auto_created=True, related_name='bootstrap4_tabs_bootstrap4tab', on_delete=models.CASCADE)),
                ('tab_style', models.CharField(max_length=255, choices=TAB_TEMPLATE_CHOICES, verbose_name='Template', default=TAB_TEMPLATE_CHOICES[0][0], help_text='This is the template that will be used for the component.')),
                ('tab_type', models.CharField(max_length=255, choices=TAB_TYPE_CHOICES, verbose_name='Type', default=TAB_TYPE_CHOICES[0][0])),
                ('tab_alignment', models.CharField(max_length=255, blank=True, choices=TAB_ALIGNMENT_CHOICES, verbose_name='Alignment')),
                ('tab_index', models.PositiveIntegerField(null=True, blank=True, verbose_name='Index', help_text='Index of element to open on page load starting at 1.')),
                ('tab_effect', models.CharField(max_length=255, blank=True, choices=TAB_EFFECT_CHOICES, verbose_name='Animation effect')),
                ('tag_type', djangocms_bootstrap4.fields.TagTypeField(max_length=255, choices=TAG_CHOICES, verbose_name='Tag type', default=TAG_CHOICES[0][0], help_text='Select the HTML tag to be used.')),
                ('attributes', djangocms_bootstrap4.fields.AttributesField(blank=True, verbose_name='Attributes', default=dict)),
            ],
            options={
                'abstract': False,
            },
            bases=('cms.cmsplugin',),
        ),
        migrations.CreateModel(
            name='Bootstrap4TabItem',
            fields=[
                ('cmsplugin_ptr', models.OneToOneField(parent_link=True, serialize=False, primary_key=True, to='cms.CMSPlugin', auto_created=True, related_name='bootstrap4_tabs_bootstrap4tabitem', on_delete=models.CASCADE)),
                ('tab_title', models.CharField(max_length=255, verbose_name='Tab title')),
                ('tag_type', djangocms_bootstrap4.fields.TagTypeField(max_length=255, choices=TAG_CHOICES, verbose_name='Tag type', default=TAG_CHOICES[0][0], help_text='Select the HTML tag to be used.')),
                ('attributes', djangocms_bootstrap4.fields.AttributesField(blank=True, verbose_name='Attributes', default=dict)),
            ],
            options={
                'abstract': False,
            },
            bases=('cms.cmsplugin',),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models

from ..constants import CAROUSEL_ASPECT_RATIO_CHOICES


class Migration(migrations.Migration):

    dependencies = [
        ('bootstrap4_carousel', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='bootstrap4carousel',
            name='carousel_aspect_ratio',
            field=models.CharField(default='', max_length=255, help_text='Determines width and height of the image according to the selected ratio.', verbose_name='Aspect ratio', choices=CAROUSEL_ASPECT_RATIO_CHOICES, blank=True),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import copy

from django.utils.translation import ugettext_lazy as _

from cms.plugin_pool import plugin_pool

from djangocms_picture.cms_plugins import PicturePlugin

from djangocms_bootstrap4.helpers import concat_classes

from .models import Bootstrap4Picture


class Bootstrap4PicturePlugin(PicturePlugin):
    """
    Content > "Image" Plugin
    https://getbootstrap.com/docs/4.0/content/images/
    """
    model = Bootstrap4Picture
    name = _('Picture / Image')
    change_form_template = 'djangocms_bootstrap4/admin/picture.html'
    module = _('Bootstrap 4')

    fieldsets = copy.deepcopy(PicturePlugin.fieldsets)
    fieldsets[0] = (
        None, {
            'fields': (
                'picture',
                'external_picture',
                ('picture_fluid', 'picture_rounded', 'picture_thumbnail'),
            )
        }
    )

    def render(self, context, instance, placeholder):
        link_classes = []
        if instance.picture_fluid:
            link_classes.append('img-fluid')
        if instance.picture_rounded:
            link_classes.append('rounded')
        if instance.picture_thumbnail:
            link_classes.append('img-thumbnail')

        classes = concat_classes(link_classes + [
            instance.attributes.get('class'),
        ])
        instance.attributes['class'] = classes

        return super(Bootstrap4PicturePlugin, self).render(
            context, instance, placeholder
        )


plugin_pool.unregister_plugin(PicturePlugin)
plugin_pool.register_plugin(Bootstrap4PicturePlugin)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models
from django.utils.translation import ugettext_lazy as _

from cms.models import CMSPlugin

from six import python_2_unicode_compatible

from djangocms_bootstrap4.fields import AttributesField, TagTypeField

from .constants import (
    TAB_ALIGNMENT_CHOICES, TAB_EFFECT_CHOICES, TAB_TEMPLATE_CHOICES,
    TAB_TYPE_CHOICES,
)


@python_2_unicode_compatible
class Bootstrap4Tab(CMSPlugin):
    """
    Components > "Navs - Tab" Plugin
    https://getbootstrap.com/docs/4.0/components/navs/
    """
    template = models.CharField(
        verbose_name=_('Template'),
        choices=TAB_TEMPLATE_CHOICES,
        default=TAB_TEMPLATE_CHOICES[0][0],
        max_length=255,
        help_text=_('This is the template that will be used for the component.'),
    )
    tab_type = models.CharField(
        verbose_name=_('Type'),
        choices=TAB_TYPE_CHOICES,
        default=TAB_TYPE_CHOICES[0][0],
        max_length=255,
    )
    tab_alignment = models.CharField(
        verbose_name=_('Alignment'),
        choices=TAB_ALIGNMENT_CHOICES,
        blank=True,
        max_length=255,
    )
    tab_index = models.PositiveIntegerField(
        verbose_name=_('Index'),
        null=True,
        blank=True,
        help_text=_('Index of element to open on page load starting at 1.'),
    )
    tab_effect = models.CharField(
        verbose_name=_('Animation effect'),
        choices=TAB_EFFECT_CHOICES,
        blank=True,
        max_length=255,
    )
    tag_type = TagTypeField()
    attributes = AttributesField()

    def __str__(self):
        return str(self.pk)

    def get_short_description(self):
        text = '({})'.format(self.tab_type)

        if self.tab_alignment:
            text += ' .{}'.format(self.tab_alignment)
        return text


@python_2_unicode_compatible
class Bootstrap4TabItem(CMSPlugin):
    """
    Components > "Navs - Tab Item" Plugin
    https://getbootstrap.com/docs/4.0/components/navs/
    """
    tab_title = models.CharField(
        verbose_name=_('Tab title'),
        max_length=255,
    )
    tag_type = TagTypeField()
    attributes = AttributesField()

    def __str__(self):
        return str(self.pk)

    def get_short_description(self):
        return self.tab_title



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models

import djangocms_bootstrap4.fields
from djangocms_bootstrap4.constants import ALIGN_CHOICES


class Migration(migrations.Migration):

    dependencies = [
        ('cms', '0016_auto_20160608_1535'),
        ('bootstrap4_content', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='Bootstrap4Figure',
            fields=[
                ('cmsplugin_ptr', models.OneToOneField(related_name='bootstrap4_content_bootstrap4figure', primary_key=True, parent_link=True, auto_created=True, to='cms.CMSPlugin', serialize=False, on_delete=models.CASCADE)),
                ('figure_caption', models.CharField(verbose_name='Caption', max_length=255)),
                ('figure_alignment', models.CharField(verbose_name='Alignment', default=ALIGN_CHOICES[0][0], choices=ALIGN_CHOICES, blank=True, max_length=255)),
                ('attributes', djangocms_bootstrap4.fields.AttributesField(verbose_name='Attributes', default=dict, blank=True)),
            ],
            options={
                'abstract': False,
            },
            bases=('cms.cmsplugin',),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations
from django.db.models import F, Func, Value
from django.db.models.functions import Concat


def forwards_func(apps, schema_editor):  # pragma: no cover
    icon = apps.get_model('bootstrap4_link', 'Bootstrap4Link')

    icon.objects.filter(icon_left__startswith='el-icon-').update(icon_left=Concat(Value('el '), 'icon_left'))
    icon.objects.filter(icon_left__startswith='flag-icon-').update(icon_left=Concat(Value('flag-icon '), 'icon_left'))
    icon.objects.filter(icon_left__startswith='fa-').update(icon_left=Concat(Value('fa '), 'icon_left'))
    icon.objects.filter(icon_left__startswith='glyphicon-').update(icon_left=Concat(Value('glyphicon '), 'icon_left'))
    icon.objects.filter(icon_left__startswith='ion-').update(icon_left=Concat(Value('ion '), 'icon_left'))
    icon.objects.filter(icon_left__startswith='map-icon-').update(icon_left=Concat(Value('map-icon '), 'icon_left'))
    icon.objects.filter(icon_left__startswith='zmdi-').update(icon_left=Concat(Value('zmdi '), 'icon_left'))
    icon.objects.filter(icon_left__startswith='wi-').update(icon_left=Concat(Value('wi '), 'icon_left'))

    icon.objects.filter(icon_right__startswith='el-icon-').update(icon_right=Concat(Value('el '), 'icon_right'))
    icon.objects.filter(icon_right__startswith='flag-icon-').update(icon_right=Concat(Value('flag-icon '), 'icon_right'))
    icon.objects.filter(icon_right__startswith='fa-').update(icon_right=Concat(Value('fa '), 'icon_right'))
    icon.objects.filter(icon_right__startswith='glyphicon-').update(icon_right=Concat(Value('glyphicon '), 'icon_right'))
    icon.objects.filter(icon_right__startswith='ion-').update(icon_right=Concat(Value('ion '), 'icon_right'))
    icon.objects.filter(icon_right__startswith='map-icon-').update(icon_right=Concat(Value('map-icon '), 'icon_right'))
    icon.objects.filter(icon_right__startswith='zmdi-').update(icon_right=Concat(Value('zmdi '), 'icon_right'))
    icon.objects.filter(icon_right__startswith='wi-').update(icon_right=Concat(Value('wi '), 'icon_right'))

def reverse_func(apps, schema_editor):  # pragma: no cover
    icon = apps.get_model('bootstrap4_link', 'Bootstrap4Link')

    icon.objects.filter(icon_left__startswith='el ').update(icon_left=Func(F('icon_left'), Value('el '), Value(''), function='replace',))
    icon.objects.filter(icon_left__startswith='flag-icon ').update(icon_left=Func(F('icon_left'), Value('flag-icon '), Value(''), function='replace',))
    icon.objects.filter(icon_left__startswith='fa ').update(icon_left=Func(F('icon_left'), Value('fa '), Value(''), function='replace',))
    icon.objects.filter(icon_left__startswith='glyphicon ').update(icon_left=Func(F('icon_left'), Value('glyphicon '), Value(''), function='replace',))
    icon.objects.filter(icon_left__startswith='map-icon ').update(icon_left=Func(F('icon_left'), Value('map-icon '), Value(''), function='replace',))
    icon.objects.filter(icon_left__startswith='zmdi ').update(icon_left=Func(F('icon_left'), Value('zmdi '), Value(''), function='replace',))
    icon.objects.filter(icon_left__startswith='wi ').update(icon_left=Func(F('icon_left'), Value('wi '), Value(''), function='replace',))

    icon.objects.filter(icon_right__startswith='el ').update(icon_right=Func(F('icon_right'), Value('el '), Value(''), function='replace',))
    icon.objects.filter(icon_right__startswith='flag-icon ').update(icon_right=Func(F('icon_right'), Value('flag-icon '), Value(''), function='replace',))
    icon.objects.filter(icon_right__startswith='fa ').update(icon_right=Func(F('icon_right'), Value('fa '), Value(''), function='replace',))
    icon.objects.filter(icon_right__startswith='glyphicon ').update(icon_right=Func(F('icon_right'), Value('glyphicon '), Value(''), function='replace',))
    icon.objects.filter(icon_right__startswith='map-icon ').update(icon_right=Func(F('icon_right'), Value('map-icon '), Value(''), function='replace',))
    icon.objects.filter(icon_right__startswith='zmdi ').update(icon_right=Func(F('icon_right'), Value('zmdi '), Value(''), function='replace',))
    icon.objects.filter(icon_right__startswith='wi ').update(icon_right=Func(F('icon_right'), Value('wi '), Value(''), function='replace',))


class Migration(migrations.Migration):

    dependencies = [
        ('bootstrap4_link', '0002_add_icons'),
    ]

    operations = [
        migrations.RunPython(forwards_func, reverse_func),
    ]



# -*- coding: utf-8 -*-
from cms.api import add_plugin
from cms.test_utils.testcases import CMSTestCase

from djangocms_bootstrap4.contrib.bootstrap4_picture.cms_plugins import (
    Bootstrap4PicturePlugin,
)

from ..fixtures import B4TestFixture
from ..helpers import get_filer_image


class B4PicturePluginTestCase(B4TestFixture, CMSTestCase):

    def setUp(self):
        super(B4PicturePluginTestCase, self).setUp()
        self.image = get_filer_image()

    def tearDown(self):
        super(B4PicturePluginTestCase, self).tearDown()
        self.image.delete()

    def test_plugin(self):
        plugin = add_plugin(
            placeholder=self.placeholder,
            plugin_type=Bootstrap4PicturePlugin.__name__,
            language=self.language,
            picture=self.image,
        )
        plugin.full_clean()
        self.page.publish(self.language)

        with self.login_user_context(self.superuser):
            response = self.client.get(self.request_url)
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, 'class="img-fluid"')

        # test picture_fluid, picture_rounded and picture_thumbnail options
        plugin = add_plugin(
            placeholder=self.placeholder,
            plugin_type=Bootstrap4PicturePlugin.__name__,
            language=self.language,
            picture=self.image,
            picture_fluid=False,
            picture_rounded=True,
            picture_thumbnail=True,
        )
        plugin.full_clean()
        self.page.publish(self.language)

        with self.login_user_context(self.superuser):
            response = self.client.get(self.request_url)
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, 'class="rounded img-thumbnail"')



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.utils.translation import ugettext_lazy as _


CARD_TYPE_CHOICES = (
    ('card', _('Card')),
    ('card-group', _('Card group')),
    ('card-deck', _('Card deck')),
    ('card-columns', _('Card columns')),
)

CARD_ALIGNMENT_CHOICES = (
    ('text-left', _('Left')),
    ('text-center', _('Center')),
    ('text-right', _('Right')),
)

CARD_INNER_TYPE_CHOICES = (
    ('card-body', _('Body')),
    ('card-header', _('Header')),
    ('card-footer', _('Footer')),
)

CARD_TAG_CHOICES = (
    ('div', 'DIV'),
    ('h1', 'H1'),
    ('h2', 'H2'),
    ('h3', 'H3'),
    ('h4', 'H4'),
    ('h5', 'H5'),
    ('h6', 'H6'),
    ('p', 'P'),
    ('small', 'SMALL'),
)



# -*- coding: utf-8 -*-
from django.test import TestCase

from djangocms_bootstrap4.contrib.bootstrap4_badge.models import (
    Bootstrap4Badge,
)


class B4BadgeModelTestCase(TestCase):

    def test_instance(self):
        instance = Bootstrap4Badge.objects.create()
        self.assertEqual(str(instance), "1")
        self.assertEqual(instance.get_short_description(), "(primary)")



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models

import djangocms_bootstrap4.fields


class Migration(migrations.Migration):

    dependencies = [
        ('bootstrap4_grid', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='bootstrap4gridcolumn',
            name='lg_offset',
            field=djangocms_bootstrap4.fields.IntegerRangeField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name='bootstrap4gridcolumn',
            name='md_offset',
            field=djangocms_bootstrap4.fields.IntegerRangeField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name='bootstrap4gridcolumn',
            name='sm_offset',
            field=djangocms_bootstrap4.fields.IntegerRangeField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name='bootstrap4gridcolumn',
            name='xl_offset',
            field=djangocms_bootstrap4.fields.IntegerRangeField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name='bootstrap4gridcolumn',
            name='xs_offset',
            field=djangocms_bootstrap4.fields.IntegerRangeField(blank=True, null=True),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models

import djangocms_bootstrap4.fields
from djangocms_bootstrap4.constants import ALIGN_CHOICES

from ..constants import CODE_TYPE_CHOICES


class Migration(migrations.Migration):

    dependencies = [
        ('cms', '0016_auto_20160608_1535'),
    ]

    operations = [
        migrations.CreateModel(
            name='Bootstrap4Blockquote',
            fields=[
                ('cmsplugin_ptr', models.OneToOneField(primary_key=True, to='cms.CMSPlugin', serialize=False, auto_created=True, related_name='bootstrap4_content_bootstrap4blockquote', parent_link=True, on_delete=models.CASCADE)),
                ('quote_content', models.TextField(verbose_name='Quote')),
                ('quote_origin', models.TextField(verbose_name='Cite', blank=True)),
                ('quote_alignment', models.CharField(max_length=255, default=ALIGN_CHOICES[0][0], choices=ALIGN_CHOICES, blank=True, verbose_name='Alignment')),
                ('attributes', djangocms_bootstrap4.fields.AttributesField(default=dict, verbose_name='Attributes', blank=True)),
            ],
            options={
                'abstract': False,
            },
            bases=('cms.cmsplugin',),
        ),
        migrations.CreateModel(
            name='Bootstrap4Code',
            fields=[
                ('cmsplugin_ptr', models.OneToOneField(primary_key=True, to='cms.CMSPlugin', serialize=False, auto_created=True, related_name='bootstrap4_content_bootstrap4code', parent_link=True, on_delete=models.CASCADE)),
                ('code_content', models.TextField(verbose_name='Code')),
                ('tag_type', models.CharField(max_length=255, default=CODE_TYPE_CHOICES[0][0], choices=CODE_TYPE_CHOICES, verbose_name='Code type')),
                ('attributes', djangocms_bootstrap4.fields.AttributesField(default=dict, verbose_name='Attributes', blank=True)),
            ],
            options={
                'abstract': False,
            },
            bases=('cms.cmsplugin',),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models
from django.utils.translation import ugettext_lazy as _

from djangocms_attributes_field import fields

from .constants import TAG_CHOICES


class AttributesField(fields.AttributesField):

    def __init__(self, *args, **kwargs):
        if 'verbose_name' not in kwargs:
            kwargs['verbose_name'] = _('Attributes')
        if 'blank' not in kwargs:
            kwargs['blank'] = True
        super(AttributesField, self).__init__(*args, **kwargs)


class TagTypeField(models.CharField):

    def __init__(self, *args, **kwargs):
        if 'verbose_name' not in kwargs:
            kwargs['verbose_name'] = _('Tag type')
        if 'choices' not in kwargs:
            kwargs['choices'] = TAG_CHOICES
        if 'default' not in kwargs:
            kwargs['default'] = TAG_CHOICES[0][0]
        if 'max_length' not in kwargs:
            kwargs['max_length'] = 255
        if 'help_text' not in kwargs:
            kwargs['help_text'] = _('Select the HTML tag to be used.')
        super(TagTypeField, self).__init__(*args, **kwargs)


class IntegerRangeField(models.IntegerField):

    def __init__(self, verbose_name=None, name=None, min_value=None, max_value=None, **kwargs):
        self.min_value, self.max_value = min_value, max_value
        models.IntegerField.__init__(self, verbose_name, name, **kwargs)

    def formfield(self, **kwargs):
        defaults = {
            'min_value': self.min_value,
            'max_value': self.max_value,
        }
        defaults.update(kwargs)
        return super(IntegerRangeField, self).formfield(**defaults)



# -*- coding: utf-8 -*-
from django.test import TestCase

from djangocms_bootstrap4.contrib.bootstrap4_link.models import Bootstrap4Link


class B4LinkModelTestCase(TestCase):

    def test_instance(self):
        instance = Bootstrap4Link.objects.create()
        self.assertEqual(str(instance), "1")



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.conf import settings


SPACER_PROPERTY_CHOICES = (
    ('m', 'margin'),
    ('p', 'padding'),
)

SPACER_SIDE_CHOICES = (
    ('', '*'),
    ('t', '*-top'),
    ('r', '*-right'),
    ('b', '*-bottom'),
    ('l', '*-left'),
    ('x', '*-left & *-right'),
    ('y', '*-top & *-bottom'),
)

SPACER_SIZE_CHOICES = getattr(
    settings,
    'DJANGOCMS_BOOTSTRAP4_SPACER_SIZES',
    (
        ('0', '* 0'),
        ('1', '* .25'),
        ('2', '* .5'),
        ('3', '* 1'),
        ('4', '* 1.5'),
        ('5', '* 3'),
    ),
)



# Generated by Django 2.1 on 2018-12-12 09:55
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('bootstrap4_picture', '0002_bootstrap4picture_use_responsive_image'),
    ]

    operations = [
        migrations.AlterField(
            model_name='bootstrap4picture',
            name='use_automatic_scaling',
            field=models.BooleanField(blank=True, default=True, help_text='Uses the placeholder dimensions to automatically calculate the size.', verbose_name='Automatic scaling'),
        ),
        migrations.AlterField(
            model_name='bootstrap4picture',
            name='use_crop',
            field=models.BooleanField(blank=True, default=False, help_text='Crops the image according to the thumbnail settings provided in the template.', verbose_name='Crop image'),
        ),
        migrations.AlterField(
            model_name='bootstrap4picture',
            name='use_no_cropping',
            field=models.BooleanField(blank=True, default=False, help_text='Outputs the raw image without cropping.', verbose_name='Use original image'),
        ),
        migrations.AlterField(
            model_name='bootstrap4picture',
            name='use_upscale',
            field=models.BooleanField(blank=True, default=False, help_text='Upscales the image to the size of the thumbnail settings in the template.', verbose_name='Upscale image'),
        ),
    ]



# -*- coding: utf-8 -*-
from django.test import TestCase

from djangocms_bootstrap4.contrib.bootstrap4_grid.models import (
    Bootstrap4GridColumn, Bootstrap4GridContainer, Bootstrap4GridRow,
)


class B4GridModelTestCase(TestCase):

    def test_grid_instance(self):
        instance = Bootstrap4GridContainer.objects.create()
        self.assertEqual(str(instance), "1")
        self.assertEqual(instance.get_short_description(), "(Container)")

    def test_row_instance(self):
        instance = Bootstrap4GridRow.objects.create()
        self.assertEqual(str(instance), "1")
        self.assertEqual(instance.get_short_description(), "(0 columns)")

    def test_column_instance(self):
        instance = Bootstrap4GridColumn.objects.create()
        self.assertEqual(str(instance), "1")
        self.assertEqual(instance.get_short_description(), "(auto) ")
        instance.xs_col = 12
        self.assertEqual(
            instance.get_short_description(),
            "(col-12) .col-12",
        )
        instance.column_type = "column"
        self.assertEqual(
            instance.get_short_description(),
            "(col-12) .column .col-12",
        )
        instance.md_col = 12
        instance.md_offset = 12
        instance.xs_offset = 12
        self.assertEqual(
            instance.get_short_description(),
            "(col-12) .column .col-12 .offset-12 .col-md-12 .offset-md-12",
        )
        instance.xs_ml = 12
        instance.md_ml = 12
        self.assertEqual(
            instance.get_short_description(),
            "(col-12) .column .col-12 .offset-12 .ml-auto .col-md-12 .offset-md-12 .ml-md-auto",
        )



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models
from django.utils.translation import ugettext_lazy as _

from cms.models import CMSPlugin

from six import python_2_unicode_compatible

from djangocms_bootstrap4.constants import COLOR_STYLE_CHOICES
from djangocms_bootstrap4.fields import AttributesField, TagTypeField

from .constants import LISTGROUP_STATE_CHOICES


@python_2_unicode_compatible
class Bootstrap4ListGroup(CMSPlugin):
    """
    Components > "List Group" Plugin
    https://getbootstrap.com/docs/4.0/components/list-group/
    """
    list_group_flush = models.BooleanField(
        verbose_name=_('List group flush'),
        default=False,
        help_text=_('Create lists of content in a card with a flush list group.')
    )
    tag_type = TagTypeField()
    attributes = AttributesField()

    def __str__(self):
        return str(self.pk)

    def get_short_description(self):
        text = ''
        if self.list_group_flush:
            text += '.list-group-flush'
        return text


@python_2_unicode_compatible
class Bootstrap4ListGroupItem(CMSPlugin):
    """
    Components > "List Group Ite" Plugin
    https://getbootstrap.com/docs/4.0/components/list-group/
    """
    list_context = models.CharField(
        verbose_name=_('Context'),
        choices=COLOR_STYLE_CHOICES,
        blank=True,
        max_length=255,
    )
    list_state = models.CharField(
        verbose_name=_('State'),
        choices=LISTGROUP_STATE_CHOICES,
        blank=True,
        max_length=255,
    )
    tag_type = TagTypeField()
    attributes = AttributesField()

    def __str__(self):
        return str(self.pk)

    def get_short_description(self):
        text = []
        if self.list_context:
            text.append('.list-group-item-{}'.format(self.list_context))
        if self.list_state:
            text.append('.{}'.format(self.list_state))
        return ' '.join(text)



# -*- coding: utf-8 -*-
# Generated by Django 1.9.13 on 2017-10-13 19:22
from __future__ import unicode_literals

import django.db.models.deletion
from django.db import migrations, models

import cms.models.fields

import djangocms_attributes_field.fields
import filer.fields.image
from djangocms_picture.models import (
    LINK_TARGET, PICTURE_ALIGNMENT, get_templates,
)


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('cms', '0016_auto_20160608_1535'),
        ('filer', '0007_auto_20161016_1055'),
    ]

    operations = [
        migrations.CreateModel(
            name='Bootstrap4Picture',
            fields=[
                ('template', models.CharField(choices=get_templates(), default=get_templates()[0][0], max_length=255, verbose_name='Template')),
                ('external_picture', models.URLField(blank=True, help_text='If provided, overrides the embedded image. Certain options such as cropping are not applicable to external images.', max_length=255, verbose_name='External image')),
                ('width', models.PositiveIntegerField(blank=True, help_text='The image width as number in pixels. Example: "720" and not "720px".', null=True, verbose_name='Width')),
                ('height', models.PositiveIntegerField(blank=True, help_text='The image height as number in pixels. Example: "720" and not "720px".', null=True, verbose_name='Height')),
                ('alignment', models.CharField(blank=True, choices=PICTURE_ALIGNMENT, help_text='Aligns the image according to the selected option.', max_length=255, verbose_name='Alignment')),
                ('caption_text', models.TextField(blank=True, help_text='Provide a description, attribution, copyright or other information.', verbose_name='Caption text')),
                ('attributes', djangocms_attributes_field.fields.AttributesField(blank=True, default=dict, verbose_name='Attributes')),
                ('link_url', models.URLField(blank=True, help_text='Wraps the image in a link to an external URL.', max_length=2040, verbose_name='External URL')),
                ('link_target', models.CharField(blank=True, choices=LINK_TARGET, max_length=255, verbose_name='Link target')),
                ('link_attributes', djangocms_attributes_field.fields.AttributesField(blank=True, default=dict, verbose_name='Link attributes')),
                ('use_automatic_scaling', models.BooleanField(default=True, help_text='Uses the placeholder dimensions to automatically calculate the size.', verbose_name='Automatic scaling')),
                ('use_no_cropping', models.BooleanField(default=False, help_text='Outputs the raw image without cropping.', verbose_name='Use original image')),
                ('use_crop', models.BooleanField(default=False, help_text='Crops the image according to the thumbnail settings provided in the template.', verbose_name='Crop image')),
                ('use_upscale', models.BooleanField(default=False, help_text='Upscales the image to the size of the thumbnail settings in the template.', verbose_name='Upscale image')),
                ('cmsplugin_ptr', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, related_name='bootstrap4_picture_bootstrap4picture', serialize=False, to='cms.CMSPlugin')),
                ('picture_fluid', models.BooleanField(default=True, help_text='Adds the .img-fluid class to make the image responsive.', verbose_name='Responsive')),
                ('picture_rounded', models.BooleanField(default=False, help_text='Adds the .rounded class for round corners.', verbose_name='Rounded')),
                ('picture_thumbnail', models.BooleanField(default=False, help_text='Adds the .img-thumbnail class.', verbose_name='Thumbnail')),
                ('link_page', cms.models.fields.PageField(blank=True, help_text='Wraps the image in a link to an internal (page) URL.', null=True, on_delete=django.db.models.deletion.SET_NULL, to='cms.Page', verbose_name='Internal URL')),
                ('picture', filer.fields.image.FilerImageField(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to='filer.Image', verbose_name='Image')),
                ('thumbnail_options', models.ForeignKey(blank=True, help_text='Overrides width, height, and crop; scales up to the provided preset dimensions.', null=True, on_delete=django.db.models.deletion.CASCADE, to='filer.ThumbnailOption', verbose_name='Thumbnail options')),
            ],
            options={
                'abstract': False,
            },
            bases=('cms.cmsplugin',),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models
from django.utils.translation import ugettext_lazy as _

from cms.models import CMSPlugin

from six import python_2_unicode_compatible

from djangocms_bootstrap4.fields import AttributesField, TagTypeField


@python_2_unicode_compatible
class Bootstrap4Jumbotron(CMSPlugin):
    """
    Components > "Jumbotron" Plugin
    https://getbootstrap.com/docs/4.0/components/jumbotron/
    """
    fluid = models.BooleanField(
        verbose_name=_('Fluid'),
        default=False,
        help_text=_('Adds the .jumbotron-fluid class.'),
    )
    tag_type = TagTypeField()
    attributes = AttributesField()

    def __str__(self):
        return str(self.pk)

    def get_short_description(self):
        text = ''
        if self.fluid:
            text = '({})'.format(_('Fluid'))
        return text



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.conf import settings
from django.utils.translation import ugettext_lazy as _


DEVICE_CHOICES = (
    ('xs', _('Extra small')),   # default <576px
    ('sm', _('Small')),         # default ≥576px
    ('md', _('Medium')),        # default ≥768px
    ('lg', _('Large')),         # default ≥992px
    ('xl', _('Extra large')),   # default ≥1200px
)
DEVICE_SIZES = tuple([size for size, name in DEVICE_CHOICES])

# Only adding block elements
TAG_CHOICES = getattr(
    settings,
    'DJANGOCMS_BOOTSTRAP4_TAG_CHOICES',
    ['div', 'section', 'article', 'header', 'footer', 'aside'],
)
TAG_CHOICES = tuple((entry, entry) for entry in TAG_CHOICES)

COLOR_STYLE_CHOICES = getattr(
    settings,
    'DJANGOCMS_BOOTSTRAP4_COLOR_STYLE_CHOICES',
    (
        ('primary', _('Primary')),
        ('secondary', _('Secondary')),
        ('success', _('Success')),
        ('danger', _('Danger')),
        ('warning', _('Warning')),
        ('info', _('Info')),
        ('light', _('Light')),
        ('dark', _('Dark')),
    )
)

ALIGN_CHOICES = (
    ('text-left', _('Left')),
    ('text-center', _('Center')),
    ('text-right', _('Right')),
)



# -*- coding: utf-8 -*-
from django.test import TestCase

from djangocms_bootstrap4.contrib.bootstrap4_media.models import (
    Bootstrap4Media, Bootstrap4MediaBody,
)


class B4MediaModelTestCase(TestCase):

    def test_media_instance(self):
        instance = Bootstrap4Media.objects.create()
        self.assertEqual(str(instance), "1")
        self.assertEqual(instance.get_short_description(), "")

    def test_media_body_instance(self):
        instance = Bootstrap4MediaBody.objects.create()
        self.assertEqual(str(instance), "1")
        self.assertEqual(instance.get_short_description(), "")



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.utils.translation import ugettext_lazy as _

from cms.plugin_base import CMSPluginBase
from cms.plugin_pool import plugin_pool

from djangocms_bootstrap4.helpers import concat_classes

from .forms import Bootstrap4CodeForm
from .models import Bootstrap4Blockquote, Bootstrap4Code, Bootstrap4Figure


class Bootstrap4CodePlugin(CMSPluginBase):
    """
    Content > "Code" Plugin
    https://getbootstrap.com/docs/4.0/content/code/
    """
    model = Bootstrap4Code
    name = _('Code')
    module = _('Bootstrap 4')
    form = Bootstrap4CodeForm
    render_template = 'djangocms_bootstrap4/code.html'
    change_form_template = 'djangocms_bootstrap4/admin/code.html'
    text_enabled = True

    fieldsets = [
        (None, {
            'fields': (
                'code_content',
                'tag_type',
            )
        }),
        (_('Advanced settings'), {
            'classes': ('collapse',),
            'fields': (
                'attributes',
            )
        }),
    ]


class Bootstrap4BlockquotePlugin(CMSPluginBase):
    """
    Content > "Blockquote" Plugin
    https://getbootstrap.com/docs/4.0/content/typography/#blockquotes
    """
    model = Bootstrap4Blockquote
    name = _('Blockquote')
    module = _('Bootstrap 4')
    render_template = 'djangocms_bootstrap4/blockquote.html'
    change_form_template = 'djangocms_bootstrap4/admin/blockquote.html'
    text_enabled = True

    fieldsets = [
        (None, {
            'fields': (
                ('quote_content', 'quote_origin'),
                'quote_alignment',
            )
        }),
        (_('Advanced settings'), {
            'classes': ('collapse',),
            'fields': (
                'attributes',
            )
        }),
    ]

    def render(self, context, instance, placeholder):
        link_classes = ['blockquote']
        if instance.quote_alignment:
            link_classes.append(instance.quote_alignment)
        classes = concat_classes(link_classes + [
            instance.attributes.get('class'),
        ])
        instance.attributes['class'] = classes

        return super(Bootstrap4BlockquotePlugin, self).render(
            context, instance, placeholder
        )


class Bootstrap4FigurePlugin(CMSPluginBase):
    """
    Content > "Figure" Plugin
    https://getbootstrap.com/docs/4.0/content/figures/
    """
    model = Bootstrap4Figure
    name = _('Figure')
    module = _('Bootstrap 4')
    render_template = 'djangocms_bootstrap4/figure.html'
    change_form_template = 'djangocms_bootstrap4/admin/figure.html'
    allow_children = True
    child_classes = ['Bootstrap4PicturePlugin']

    fieldsets = [
        (None, {
            'fields': (
                'figure_caption',
                'figure_alignment',
            )
        }),
        (_('Advanced settings'), {
            'classes': ('collapse',),
            'fields': (
                'attributes',
            )
        }),
    ]

    def render(self, context, instance, placeholder):
        classes = concat_classes([
            'figure',
            instance.attributes.get('class'),
        ])
        instance.attributes['class'] = classes

        return super(Bootstrap4FigurePlugin, self).render(
            context, instance, placeholder
        )


plugin_pool.register_plugin(Bootstrap4CodePlugin)
plugin_pool.register_plugin(Bootstrap4BlockquotePlugin)
plugin_pool.register_plugin(Bootstrap4FigurePlugin)



# -*- coding: utf-8 -*-
from cms.api import add_plugin
from cms.test_utils.testcases import CMSTestCase

from djangocms_bootstrap4.contrib.bootstrap4_carousel.cms_plugins import (
    Bootstrap4CarouselPlugin, Bootstrap4CarouselSlidePlugin,
)

from ..fixtures import B4TestFixture
from ..helpers import get_filer_image


class B4CarouselPluginTestCase(B4TestFixture, CMSTestCase):

    def setUp(self):
        super(B4CarouselPluginTestCase, self).setUp()
        self.image = get_filer_image()

    def tearDown(self):
        super(B4CarouselPluginTestCase, self).tearDown()
        self.image.delete()

    def test_carousel_plugin(self):
        plugin = add_plugin(
            placeholder=self.placeholder,
            plugin_type=Bootstrap4CarouselPlugin.__name__,
            language=self.language,
        )
        plugin.full_clean()
        self.page.publish(self.language)

        with self.login_user_context(self.superuser):
            response = self.client.get(self.request_url)
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, '<div class="carousel slide"')
        self.assertContains(response, 'data-interval="5000"')
        self.assertContains(response, 'data-keyboard="true"')
        self.assertContains(response, 'data-pause="hover"')
        self.assertContains(response, 'data-ride="carousel"')
        self.assertContains(response, 'data-wrap="true"')

    def test_carousel_slide_plugin(self):
        row = add_plugin(
            placeholder=self.placeholder,
            plugin_type=Bootstrap4CarouselPlugin.__name__,
            language=self.language,
        )
        plugin = add_plugin(
            target=row,
            placeholder=self.placeholder,
            plugin_type=Bootstrap4CarouselSlidePlugin.__name__,
            language=self.language,
            carousel_image=self.image,
        )
        plugin.full_clean()
        self.page.publish(self.language)

        with self.login_user_context(self.superuser):
            response = self.client.get(self.request_url)
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, '<div class="carousel slide"')

        # testing aspect ratio variant
        # also testing multiply entries
        row = add_plugin(
            placeholder=self.placeholder,
            plugin_type=Bootstrap4CarouselPlugin.__name__,
            language=self.language,
            carousel_aspect_ratio="16x9",
        )
        add_plugin(
            target=row,
            placeholder=self.placeholder,
            plugin_type=Bootstrap4CarouselSlidePlugin.__name__,
            language=self.language,
            carousel_image=self.image,
        )
        add_plugin(
            target=row,
            placeholder=self.placeholder,
            plugin_type=Bootstrap4CarouselSlidePlugin.__name__,
            language=self.language,
            carousel_image=self.image,
        )
        self.page.publish(self.language)

        with self.login_user_context(self.superuser):
            response = self.client.get(self.request_url)
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, '<div class="carousel slide"')



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.utils.translation import ugettext_lazy as _

from cms.plugin_base import CMSPluginBase
from cms.plugin_pool import plugin_pool

from djangocms_bootstrap4.helpers import get_plugin_template

from .constants import TAB_TEMPLATE_CHOICES
from .models import Bootstrap4Tab, Bootstrap4TabItem


class Bootstrap4TabPlugin(CMSPluginBase):
    """
    Components > "Navs - Tab" Plugin
    https://getbootstrap.com/docs/4.0/components/navs/
    """
    model = Bootstrap4Tab
    name = _('Tabs')
    module = _('Bootstrap 4')
    change_form_template = 'djangocms_bootstrap4/admin/tabs.html'
    allow_children = True
    child_classes = ['Bootstrap4TabItemPlugin']

    fieldsets = [
        (None, {
            'fields': (
                ('tab_type', 'tab_alignment'),
                ('tab_index', 'tab_effect'),
            )
        }),
        (_('Advanced settings'), {
            'classes': ('collapse',),
            'fields': (
                'template',
                'tag_type',
                'attributes',
            )
        }),
    ]

    def get_render_template(self, context, instance, placeholder):
        return get_plugin_template(
            instance, 'tabs', 'tabs', TAB_TEMPLATE_CHOICES
        )


class Bootstrap4TabItemPlugin(CMSPluginBase):
    """
    Components > "Navs - Tab Item" Plugin
    https://getbootstrap.com/docs/4.0/components/navs/
    """
    model = Bootstrap4TabItem
    name = _('Tab item')
    module = _('Bootstrap 4')
    change_form_template = 'djangocms_bootstrap4/admin/tabs.html'
    allow_children = True
    parent_classes = ['Bootstrap4TabPlugin']

    fieldsets = [
        (None, {
            'fields': (
                'tab_title',
            )
        }),
        (_('Advanced settings'), {
            'classes': ('collapse',),
            'fields': (
                'tag_type',
                'attributes',
            )
        }),
    ]

    def get_render_template(self, context, instance, placeholder):
        return get_plugin_template(
            instance.parent.get_plugin_instance()[0],
            'tabs',
            'item',
            TAB_TEMPLATE_CHOICES,
        )


plugin_pool.register_plugin(Bootstrap4TabPlugin)
plugin_pool.register_plugin(Bootstrap4TabItemPlugin)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.11 on 2018-06-10 11:06
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('bootstrap4_tabs', '0001_initial'),
    ]

    operations = [
        migrations.RenameField(
            model_name='bootstrap4tab',
            old_name='tab_style',
            new_name='template',
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.13 on 2017-10-15 10:47
from __future__ import unicode_literals

import django.db.models.deletion
from django.db import migrations, models

import djangocms_bootstrap4.fields
from djangocms_bootstrap4.constants import DEVICE_CHOICES, TAG_CHOICES

from ..constants import (
    SPACER_PROPERTY_CHOICES, SPACER_SIDE_CHOICES, SPACER_SIZE_CHOICES,
)


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('cms', '0016_auto_20160608_1535'),
    ]

    operations = [
        migrations.CreateModel(
            name='Bootstrap4Spacing',
            fields=[
                ('cmsplugin_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, related_name='bootstrap4_utilities_bootstrap4spacing', serialize=False, to='cms.CMSPlugin')),
                ('space_property', models.CharField(choices=SPACER_PROPERTY_CHOICES, default=SPACER_PROPERTY_CHOICES[0][0], max_length=255, verbose_name='Property')),
                ('space_sides', models.CharField(blank=True, choices=SPACER_SIDE_CHOICES, default=SPACER_SIDE_CHOICES[0][0], max_length=255, verbose_name='Sides')),
                ('space_size', models.CharField(choices=SPACER_SIZE_CHOICES, default=SPACER_SIZE_CHOICES[0][0], max_length=255, verbose_name='Size')),
                ('space_device', models.CharField(blank=True, choices=DEVICE_CHOICES, max_length=255, verbose_name='Device')),
                ('tag_type', djangocms_bootstrap4.fields.TagTypeField(choices=TAG_CHOICES, default=TAG_CHOICES[0][0], help_text='Select the HTML tag to be used.', max_length=255, verbose_name='Tag type')),
                ('attributes', djangocms_bootstrap4.fields.AttributesField(blank=True, default=dict, verbose_name='Attributes')),
            ],
            options={
                'abstract': False,
            },
            bases=('cms.cmsplugin',),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models
from django.utils.translation import ugettext_lazy as _

from djangocms_icon.fields import Icon
from djangocms_link.models import AbstractLink
from six import python_2_unicode_compatible

from djangocms_bootstrap4.constants import COLOR_STYLE_CHOICES

from .constants import LINK_CHOICES, LINK_SIZE_CHOICES


# 'link' type is added manually as it is only required for this plugin
COLOR_STYLE_CHOICES = (
    ('link', _('Link')),
) + COLOR_STYLE_CHOICES


@python_2_unicode_compatible
class Bootstrap4Link(AbstractLink):
    """
    Components > "Button" Plugin
    https://getbootstrap.com/docs/4.0/components/buttons/
    """
    link_type = models.CharField(
        verbose_name=_('Type'),
        choices=LINK_CHOICES,
        default=LINK_CHOICES[0][0],
        max_length=255,
        help_text=_('Adds either the .btn-* or .text-* classes.'),
    )
    link_context = models.CharField(
        verbose_name=_('Context'),
        choices=COLOR_STYLE_CHOICES,
        blank=True,
        max_length=255,
    )
    link_size = models.CharField(
        verbose_name=_('Size'),
        choices=LINK_SIZE_CHOICES,
        blank=True,
        max_length=255,
    )
    link_outline = models.BooleanField(
        verbose_name=_('Outline'),
        default=False,
        help_text=_('Applies the .btn-outline class to the elements.'),
    )
    link_block = models.BooleanField(
        verbose_name=_('Block'),
        default=False,
        help_text=_('Extends the button to the width of its container.'),
    )
    icon_left = Icon(
        verbose_name=_('Icon left'),
    )
    icon_right = Icon(
        verbose_name=_('Icon right'),
    )

    def __str__(self):
        return str(self.pk)



# -*- coding: utf-8 -*-
from cms.api import add_plugin
from cms.test_utils.testcases import CMSTestCase

from djangocms_bootstrap4.contrib.bootstrap4_card.cms_plugins import (
    Bootstrap4CardInnerPlugin, Bootstrap4CardPlugin,
)

from ..fixtures import B4TestFixture


class B4CardPluginTestCase(B4TestFixture, CMSTestCase):

    def test_card_plugin(self):
        plugin = add_plugin(
            placeholder=self.placeholder,
            plugin_type=Bootstrap4CardPlugin.__name__,
            language=self.language,
        )
        plugin.full_clean()
        self.page.publish(self.language)

        with self.login_user_context(self.superuser):
            response = self.client.get(self.request_url)
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, '<div class="card">')

        # add card type
        plugin = add_plugin(
            placeholder=self.placeholder,
            plugin_type=Bootstrap4CardPlugin.__name__,
            language=self.language,
            card_type="card-columns",
            card_context="transparent",
            card_outline=True,
            card_alignment="text-right",
            card_text_color="white",
        )
        plugin.full_clean()
        self.page.publish(self.language)

        with self.login_user_context(self.superuser):
            response = self.client.get(self.request_url)
        self.assertEqual(response.status_code, 200)
        self.assertContains(
            response,
            '<div class="card-columns border-transparent text-right text-white">',
        )

        # special case when card outline is given but not card context
        plugin = add_plugin(
            placeholder=self.placeholder,
            plugin_type=Bootstrap4CardPlugin.__name__,
            language=self.language,
            card_type="card-group",
            card_context="transparent",
        )
        plugin.full_clean()
        self.page.publish(self.language)

        with self.login_user_context(self.superuser):
            response = self.client.get(self.request_url)
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, '<div class="card-group bg-transparent">')

    def test_card_inner_plugin(self):
        plugin = add_plugin(
            placeholder=self.placeholder,
            plugin_type=Bootstrap4CardInnerPlugin.__name__,
            language=self.language,
        )
        plugin.full_clean()
        self.page.publish(self.language)

        with self.login_user_context(self.superuser):
            response = self.client.get(self.request_url)
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, '<div class="card-body">')

        # add inner type
        plugin = add_plugin(
            placeholder=self.placeholder,
            plugin_type=Bootstrap4CardInnerPlugin.__name__,
            language=self.language,
            inner_type="card-footer",
        )
        plugin.full_clean()
        self.page.publish(self.language)

        with self.login_user_context(self.superuser):
            response = self.client.get(self.request_url)
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, '<div class="card-footer">')



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.utils.translation import ugettext_lazy as _

from cms.plugin_base import CMSPluginBase
from cms.plugin_pool import plugin_pool

from djangocms_bootstrap4.helpers import concat_classes

from .models import Bootstrap4Media, Bootstrap4MediaBody


class Bootstrap4MediaPlugin(CMSPluginBase):
    """
    Layout > "Media" Plugin
    http://getbootstrap.com/docs/4.0/layout/media-object/
    """
    model = Bootstrap4Media
    name = _('Media')
    module = _('Bootstrap 4')
    render_template = 'djangocms_bootstrap4/media.html'
    change_form_template = 'djangocms_bootstrap4/admin/media.html'
    allow_children = True

    fieldsets = [
        (_('Advanced settings'), {
            'classes': ('collapse',),
            'fields': (
                'tag_type',
                'attributes',
            )
        }),
    ]

    def render(self, context, instance, placeholder):
        classes = concat_classes([
            'media',
            instance.attributes.get('class'),
        ])
        instance.attributes['class'] = classes

        return super(Bootstrap4MediaPlugin, self).render(
            context, instance, placeholder
        )


class Bootstrap4MediaBodyPlugin(CMSPluginBase):
    """
    Layout > "Media body" Plugin
    http://getbootstrap.com/docs/4.0/layout/media-object/
    """
    model = Bootstrap4MediaBody
    name = _('Media body')
    module = _('Bootstrap 4')
    render_template = 'djangocms_bootstrap4/media-body.html'
    change_form_template = 'djangocms_bootstrap4/admin/media.html'
    allow_children = True
    parent_classes = ['Bootstrap4MediaPlugin']

    fieldsets = [
        (_('Advanced settings'), {
            'classes': ('collapse',),
            'fields': (
                'tag_type',
                'attributes',
            )
        }),
    ]

    def render(self, context, instance, placeholder):
        classes = concat_classes([
            'media-body',
            instance.attributes.get('class'),
        ])
        instance.attributes['class'] = classes

        return super(Bootstrap4MediaBodyPlugin, self).render(
            context, instance, placeholder
        )


plugin_pool.register_plugin(Bootstrap4MediaPlugin)
plugin_pool.register_plugin(Bootstrap4MediaBodyPlugin)



# -*- coding: utf-8 -*-
from django.test import TestCase

from djangocms_bootstrap4.contrib.bootstrap4_listgroup.models import (
    Bootstrap4ListGroup, Bootstrap4ListGroupItem,
)


class B4ListGroupModelTestCase(TestCase):

    def test_list_group_instance(self):
        instance = Bootstrap4ListGroup.objects.create()
        self.assertEqual(str(instance), "1")
        self.assertEqual(instance.get_short_description(), "")
        instance.list_group_flush = True
        self.assertEqual(instance.get_short_description(), ".list-group-flush")

    def test_list_group_item_instance(self):
        instance = Bootstrap4ListGroupItem.objects.create()
        self.assertEqual(str(instance), "1")
        self.assertEqual(instance.get_short_description(), "")
        instance.list_context = "primary"
        self.assertEqual(instance.get_short_description(), ".list-group-item-primary")
        instance.list_state = "active"
        self.assertEqual(instance.get_short_description(), ".list-group-item-primary .active")



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models

import djangocms_bootstrap4.fields


def migrate_column_size(apps, schema_editor):  # pragma: no cover
    Column = apps.get_model('bootstrap4_grid', 'Bootstrap4GridColumn')
    plugins = Column.objects.all()

    for plugin in plugins:
        if not plugin.xs_col and plugin.column_size:
            plugin.xs_col = plugin.column_size
            plugin.save()

def migrate_column_size_back(apps, schema_editor):  # pragma: no cover
    Column = apps.get_model('bootstrap4_grid', 'Bootstrap4GridColumn')
    plugins = Column.objects.all()

    for plugin in plugins:
        if plugin.xs_col:
            plugin.column_size = plugin.xs_col
            plugin.save()


class Migration(migrations.Migration):

    dependencies = [
        ('bootstrap4_grid', '0002_auto_20180709_0808'),
    ]

    operations = [
        migrations.RunPython(migrate_column_size, migrate_column_size_back)
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from cms.models import CMSPlugin

from six import python_2_unicode_compatible

from djangocms_bootstrap4.fields import AttributesField, TagTypeField


@python_2_unicode_compatible
class Bootstrap4Media(CMSPlugin):
    """
    Layout > "Media" Plugin
    http://getbootstrap.com/docs/4.0/layout/media-object/
    """
    tag_type = TagTypeField()
    attributes = AttributesField()

    def __str__(self):
        return str(self.pk)

    def get_short_description(self):
        return ''


@python_2_unicode_compatible
class Bootstrap4MediaBody(CMSPlugin):
    """
    Layout > "Media body" Plugin
    http://getbootstrap.com/docs/4.0/layout/media-object/
    """
    tag_type = TagTypeField()
    attributes = AttributesField()

    def __str__(self):
        return str(self.pk)

    def get_short_description(self):
        return ''



# -*- coding: utf-8 -*-
from django.test import TestCase

from djangocms_bootstrap4.contrib.bootstrap4_picture.models import (
    Bootstrap4Picture,
)


class B4PictureModelTestCase(TestCase):

    def test_instance(self):
        instance = Bootstrap4Picture.objects.create()
        self.assertEqual(str(instance), "1")



# -*- coding: utf-8 -*-
# Generated by Django 1.11.15 on 2018-09-28 17:18
from __future__ import unicode_literals

from django.db import migrations, models

from djangocms_picture.models import RESPONSIVE_IMAGE_CHOICES


class Migration(migrations.Migration):

    dependencies = [
        ('bootstrap4_picture', '0001_initial'),
        ('djangocms_picture', '0008_picture_use_responsive_image'),
    ]

    operations = [
        migrations.AddField(
            model_name='Bootstrap4Picture',
            name='use_responsive_image',
            field=models.CharField(choices=RESPONSIVE_IMAGE_CHOICES, default='inherit', help_text='Uses responsive image technique to choose better image to display based upon screen viewport. This configuration only applies to uploaded images (external pictures will not be affected). ', max_length=7, verbose_name='Use responsive image'),
        ),
    ]



# -*- coding: utf-8 -*-
from django.test import TestCase

from djangocms_bootstrap4.contrib.bootstrap4_collapse.models import (
    Bootstrap4Collapse, Bootstrap4CollapseContainer, Bootstrap4CollapseTrigger,
)


class B4CollapseModelTestCase(TestCase):

    def test_collapse_instance(self):
        instance = Bootstrap4Collapse.objects.create()
        self.assertEqual(str(instance), "1")
        self.assertEqual(instance.get_short_description(), "(collapse-1)")

    def test_collapse_trigger_instance(self):
        instance = Bootstrap4CollapseTrigger.objects.create()
        self.assertEqual(str(instance), "1")
        self.assertEqual(instance.get_short_description(), "()")

    def test_collapse_container_instance(self):
        instance = Bootstrap4CollapseContainer.objects.create()
        self.assertEqual(str(instance), "1")
        self.assertEqual(instance.get_short_description(), "()")



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.utils.translation import ugettext_lazy as _


LISTGROUP_STATE_CHOICES = (
    ('active', _('Active')),
    ('disabled', _('Disabled')),
)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.utils.translation import ugettext_lazy as _

from cms.plugin_base import CMSPluginBase
from cms.plugin_pool import plugin_pool

from djangocms_bootstrap4.helpers import concat_classes

from .models import (
    Bootstrap4Collapse, Bootstrap4CollapseContainer, Bootstrap4CollapseTrigger,
)


class Bootstrap4CollapsePlugin(CMSPluginBase):
    """
    Component > "Collapse" Plugin
    https://getbootstrap.com/docs/4.0/components/collapse/
    """
    model = Bootstrap4Collapse
    name = _('Collapse')
    module = _('Bootstrap 4')
    render_template = 'djangocms_bootstrap4/collapse.html'
    change_form_template = 'djangocms_bootstrap4/admin/collapse.html'
    allow_children = True
    child_classes = [
        'Bootstrap4CollapseTriggerPlugin',
        'Bootstrap4CollapseContainerPlugin',
        'Bootstrap4LinkPlugin',
        'Bootstrap4CardPlugin',
        'Bootstrap4SpacingPlugin',
        'Bootstrap4GridRowPlugin',
    ]

    fieldsets = [
        (_('Advanced settings'), {
            'classes': ('collapse',),
            'fields': (
                'siblings',
                'tag_type',
                'attributes',
            )
        }),
    ]


class Bootstrap4CollapseTriggerPlugin(CMSPluginBase):
    """
    Component > "Collapse" Plugin
    https://getbootstrap.com/docs/4.0/components/collapse/
    """
    model = Bootstrap4CollapseTrigger
    name = _('Collapse trigger')
    module = _('Bootstrap 4')
    render_template = 'djangocms_bootstrap4/collapse-trigger.html'
    allow_children = True
    parent_classes = [
        'Bootstrap4CardPlugin',
        'Bootstrap4CardInnerPlugin',
        'Bootstrap4CollapsePlugin',
        'Bootstrap4GridColumnPlugin',
    ]

    fieldsets = [
        (None, {
            'fields': (
                'identifier',
            )
        }),
        (_('Advanced settings'), {
            'classes': ('collapse',),
            'fields': (
                'tag_type',
                'attributes',
            )
        }),
    ]


class Bootstrap4CollapseContainerPlugin(CMSPluginBase):
    """
    Component > "Collapse Container" Plugin
    https://getbootstrap.com/docs/4.0/components/collapse/
    """
    model = Bootstrap4CollapseContainer
    name = _('Collapse container')
    module = _('Bootstrap 4')
    render_template = 'djangocms_bootstrap4/collapse-container.html'
    allow_children = True
    parent_classes = [
        'Bootstrap4CardPlugin',
        'Bootstrap4CardInnerPlugin',
        'Bootstrap4CollapsePlugin',
        'Bootstrap4GridColumnPlugin',
    ]

    fieldsets = [
        (None, {
            'fields': (
                'identifier',
            )
        }),
        (_('Advanced settings'), {
            'classes': ('collapse',),
            'fields': (
                'tag_type',
                'attributes',
            )
        }),
    ]

    def render(self, context, instance, placeholder):
        classes = concat_classes([
            'collapse',
            instance.attributes.get('class'),
        ])
        instance.attributes['class'] = classes

        return super(Bootstrap4CollapseContainerPlugin, self).render(
            context, instance, placeholder
        )


plugin_pool.register_plugin(Bootstrap4CollapsePlugin)
plugin_pool.register_plugin(Bootstrap4CollapseTriggerPlugin)
plugin_pool.register_plugin(Bootstrap4CollapseContainerPlugin)



# -*- coding: utf-8 -*-
from django.forms.models import ModelForm
from django.forms.widgets import Textarea


class Bootstrap4CodeForm(ModelForm):
    class Meta:
        # When used inside djangocms-text-ckeditor
        # this causes the label field to be prefilled with the selected text.
        widgets = {
            'code_content': Textarea(attrs={'class': 'js-ckeditor-use-selected-text'}),
        }



# -*- coding: utf-8 -*-
from django.test import TestCase

from djangocms_bootstrap4.contrib.bootstrap4_card.models import (
    Bootstrap4Card, Bootstrap4CardInner,
)


class B4CardModelTestCase(TestCase):

    def test_card_instance(self):
        instance = Bootstrap4Card.objects.create()
        self.assertEqual(str(instance), "1")
        self.assertEqual(instance.get_short_description(), "(card)")
        instance.card_context = "primary"
        self.assertEqual(instance.get_short_description(), "(card) .bg-primary")
        instance.card_outline = True
        self.assertEqual(instance.get_short_description(), "(card) .border-primary")
        instance.card_alignment = "center"
        self.assertEqual(instance.get_short_description(), "(card) .border-primary .center")

    def test_card_inner_instance(self):
        instance = Bootstrap4CardInner.objects.create()
        self.assertEqual(str(instance), "1")
        self.assertEqual(instance.get_short_description(), "(card-body)")



# Generated by Django 2.2.2 on 2019-07-03 08:31

import django.db.models.deletion
from django.db import migrations, models

import djangocms_link.validators
import filer.fields.file


class Migration(migrations.Migration):

    dependencies = [
        ('filer', '0011_auto_20190418_0137'),
        ('bootstrap4_link', '0003_icon_updates'),
    ]

    operations = [
        migrations.AddField(
            model_name='bootstrap4link',
            name='file_link',
            field=filer.fields.file.FilerFileField(blank=True, help_text='If provided links a file from the filer app.', null=True, on_delete=django.db.models.deletion.SET_NULL, to='filer.File', verbose_name='File link'),
        ),
        migrations.AlterField(
            model_name='bootstrap4link',
            name='external_link',
            field=models.CharField(blank=True, help_text='Provide a link to an external source.', max_length=2040, validators=[djangocms_link.validators.IntranetURLValidator(intranet_host_re=None)], verbose_name='External link'),
        ),
    ]



# -*- coding: utf-8 -*-
from django.test import TestCase

from djangocms_bootstrap4.contrib.bootstrap4_carousel.models import (
    Bootstrap4Carousel, Bootstrap4CarouselSlide,
)

from ..helpers import get_filer_image


class B4CarouselModelTestCase(TestCase):

    def setUp(self):
        super(B4CarouselModelTestCase, self).setUp()
        self.image = get_filer_image()

    def tearDown(self):
        super(B4CarouselModelTestCase, self).tearDown()
        self.image.delete()

    def test_carousel_instance(self):
        instance = Bootstrap4Carousel.objects.create()
        self.assertEqual(str(instance), "1")
        self.assertEqual(
            instance.get_short_description(),
            "(default) Interval: 5000, Controls: True, Indicators: True, "
            "Keyboard: True, Pause: hover, Ride: carouselWrap: True",
        )

    def test_carousel_slide_instance(self):
        instance = Bootstrap4CarouselSlide.objects.create()
        instance.clean()
        self.assertEqual(str(instance), "1")
        self.assertEqual(instance.get_short_description(), "")
        # test carousel content strings
        instance.carousel_image = self.image
        instance.carousel_content = "hello world"
        self.assertEqual(
            instance.get_short_description(),
            "test_file.jpg (hello world)",
        )
        instance.carousel_content = "hello world" + "#" * 100
        self.assertEqual(
            instance.get_short_description(),
            "test_file.jpg (hello world" + "#" * 89 + "...)",
        )
        self.assertEqual(instance.get_link(), "")
        # test image ouput options
        instance.carousel_content = None
        instance.carousel_image = get_filer_image(name="image")
        self.assertEqual(instance.get_short_description(), "image")
        instance.carousel_image = get_filer_image(original_filename=False)
        self.assertEqual(instance.get_short_description(), "Image")



# -*- coding: utf-8 -*-
from cms.api import add_plugin
from cms.test_utils.testcases import CMSTestCase

from djangocms_bootstrap4.contrib.bootstrap4_tabs.cms_plugins import (
    Bootstrap4TabItemPlugin, Bootstrap4TabPlugin,
)

from ..fixtures import B4TestFixture


class B4TabsPluginTestCase(B4TestFixture, CMSTestCase):

    def test_tab_plugin(self):
        plugin = add_plugin(
            placeholder=self.placeholder,
            plugin_type=Bootstrap4TabPlugin.__name__,
            language=self.language,
        )
        plugin.full_clean()
        self.page.publish(self.language)

        with self.login_user_context(self.superuser):
            response = self.client.get(self.request_url)
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, 'nav')
        self.assertContains(response, 'nav-tabs')

    def test_tab_item_plugin(self):
        parent = add_plugin(
            placeholder=self.placeholder,
            plugin_type=Bootstrap4TabPlugin.__name__,
            language=self.language,
        )
        plugin = add_plugin(
            target=parent,
            placeholder=self.placeholder,
            plugin_type=Bootstrap4TabItemPlugin.__name__,
            language=self.language,
            tab_title="tab title",
        )
        plugin.full_clean()
        self.page.publish(self.language)

        with self.login_user_context(self.superuser):
            response = self.client.get(self.request_url)
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, '<div class="tab-content">')
        self.assertContains(response, 'tab-pane')



# -*- coding: utf-8 -*-
# Generated by Django 1.9.13 on 2017-10-15 21:05
from __future__ import unicode_literals

import django.db.models.deletion
from django.db import migrations, models

import djangocms_attributes_field.fields
import djangocms_link.validators
import djangocms_text_ckeditor.fields
import filer.fields.image
from djangocms_link.models import TARGET_CHOICES, get_templates

import djangocms_bootstrap4.fields
from djangocms_bootstrap4.constants import TAG_CHOICES

from ..constants import (
    CAROUSEL_PAUSE_CHOICES, CAROUSEL_RIDE_CHOICES, CAROUSEL_TEMPLATE_CHOICES,
)


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('cms', '0016_auto_20160608_1535'),
        ('filer', '0007_auto_20161016_1055'),
    ]

    operations = [
        migrations.CreateModel(
            name='Bootstrap4Carousel',
            fields=[
                ('cmsplugin_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, related_name='bootstrap4_carousel_bootstrap4carousel', serialize=False, to='cms.CMSPlugin')),
                ('carousel_style', models.CharField(choices=CAROUSEL_TEMPLATE_CHOICES, default=CAROUSEL_TEMPLATE_CHOICES[0][0], help_text='This is the template that will be used for the component.', max_length=255, verbose_name='Template')),
                ('carousel_interval', models.IntegerField(default=5000, help_text='The amount of time to delay between automatically cycling an item. If false, carousel will not automatically cycle.', verbose_name='Interval')),
                ('carousel_controls', models.BooleanField(default=True, help_text='Adding in the previous and next controls.', verbose_name='Controls')),
                ('carousel_indicators', models.BooleanField(default=True, help_text='Adding in the indicators to the carousel.', verbose_name='Indicators')),
                ('carousel_keyboard', models.BooleanField(default=True, help_text='Whether the carousel should react to keyboard events.', verbose_name='Keyboard')),
                ('carousel_pause', models.CharField(choices=CAROUSEL_PAUSE_CHOICES, default=CAROUSEL_PAUSE_CHOICES[0][0], help_text='If set to "hover", pauses the cycling of the carousel on "mouseenter" and resumes the cycling of the carousel on "mouseleave". If set to "false", hovering over the carousel won\'t pause it.', max_length=255, verbose_name='Pause')),
                ('carousel_ride', models.CharField(choices=CAROUSEL_RIDE_CHOICES, default=CAROUSEL_RIDE_CHOICES[0][0], help_text='Autoplays the carousel after the user manually cycles the first item. If "carousel", autoplays the carousel on load.', max_length=255, verbose_name='Ride')),
                ('carousel_wrap', models.BooleanField(default=True, help_text='Whether the carousel should cycle continuously or have hard stops.', verbose_name='Wrap')),
                ('tag_type', djangocms_bootstrap4.fields.TagTypeField(choices=TAG_CHOICES, default=TAG_CHOICES[0][0], help_text='Select the HTML tag to be used.', max_length=255, verbose_name='Tag type')),
                ('attributes', djangocms_bootstrap4.fields.AttributesField(blank=True, default=dict, verbose_name='Attributes')),
            ],
            options={
                'abstract': False,
            },
            bases=('cms.cmsplugin',),
        ),
        migrations.CreateModel(
            name='Bootstrap4CarouselSlide',
            fields=[
                ('template', models.CharField(choices=get_templates(), default=get_templates()[0][0], max_length=255, verbose_name='Template')),
                ('name', models.CharField(blank=True, max_length=255, verbose_name='Display name')),
                ('external_link', models.URLField(blank=True, help_text='Provide a valid URL to an external website.', max_length=2040, validators=[djangocms_link.validators.IntranetURLValidator(intranet_host_re=None)], verbose_name='External link')),
                ('anchor', models.CharField(blank=True, help_text='Appends the value only after the internal or external link. Do <em>not</em> include a preceding "#" symbol.', max_length=255, verbose_name='Anchor')),
                ('mailto', models.EmailField(blank=True, max_length=255, verbose_name='Email address')),
                ('phone', models.CharField(blank=True, max_length=255, verbose_name='Phone')),
                ('target', models.CharField(blank=True, choices=TARGET_CHOICES, max_length=255, verbose_name='Target')),
                ('attributes', djangocms_attributes_field.fields.AttributesField(blank=True, default=dict, verbose_name='Attributes')),
                ('cmsplugin_ptr', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, related_name='bootstrap4_carousel_bootstrap4carouselslide', serialize=False, to='cms.CMSPlugin')),
                ('carousel_content', djangocms_text_ckeditor.fields.HTMLField(blank=True, default='', help_text='Content may also be added using child plugins.', verbose_name='Content')),
                ('tag_type', djangocms_bootstrap4.fields.TagTypeField(choices=TAG_CHOICES, default=TAG_CHOICES[0][0], help_text='Select the HTML tag to be used.', max_length=255, verbose_name='Tag type')),
                ('carousel_image', filer.fields.image.FilerImageField(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to='filer.Image', verbose_name='Slide image')),
                ('internal_link', models.ForeignKey(blank=True, help_text='If provided, overrides the external link.', null=True, on_delete=django.db.models.deletion.SET_NULL, to='cms.Page', verbose_name='Internal link')),
            ],
            options={
                'abstract': False,
            },
            bases=('cms.cmsplugin',),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.conf import settings
from django.utils.translation import ugettext_lazy as _


LINK_CHOICES = (
    ('link', _('Link')),
    ('btn', _('Button')),
)

LINK_SIZE_CHOICES = (
    ('btn-sm', _('Small')),
    ('', _('Medium')),
    ('btn-lg', _('Large')),
)

USE_LINK_ICONS = getattr(
    settings,
    'DJANGOCMS_BOOTSTRAP4_USE_ICONS',
    True,
)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.utils.translation import ugettext_lazy as _

from cms.plugin_base import CMSPluginBase
from cms.plugin_pool import plugin_pool

from djangocms_bootstrap4.helpers import concat_classes

from .models import Bootstrap4ListGroup, Bootstrap4ListGroupItem


class Bootstrap4ListGroupPlugin(CMSPluginBase):
    """
    Components > "List Group" Plugin
    https://getbootstrap.com/docs/4.0/components/list-group/
    """
    model = Bootstrap4ListGroup
    name = _('List group')
    module = _('Bootstrap 4')
    render_template = 'djangocms_bootstrap4/list-group.html'
    change_form_template = 'djangocms_bootstrap4/admin/list-group.html'
    allow_children = True
    child_classes = ['Bootstrap4ListGroupItemPlugin', 'Bootstrap4LinkPlugin']
    # TODO consider linking to tab-content

    fieldsets = [
        (None, {
            'fields': (
                'list_group_flush',
            )
        }),
        (_('Advanced settings'), {
            'classes': ('collapse',),
            'fields': (
                'tag_type',
                'attributes',
            )
        }),
    ]

    def render(self, context, instance, placeholder):
        link_classes = ['list-group']
        if instance.list_group_flush:
            link_classes.append('list-group-flush')

        classes = concat_classes(link_classes + [
            instance.attributes.get('class'),
        ])
        instance.attributes['class'] = classes

        return super(Bootstrap4ListGroupPlugin, self).render(
            context, instance, placeholder
        )


class Bootstrap4ListGroupItemPlugin(CMSPluginBase):
    """
    Components > "List Group Item" Plugin
    https://getbootstrap.com/docs/4.0/components/list-group/
    """
    model = Bootstrap4ListGroupItem
    name = _('List item')
    module = _('Bootstrap 4')
    render_template = 'djangocms_bootstrap4/list-group-item.html'
    change_form_template = 'djangocms_bootstrap4/admin/list-group.html'
    allow_children = True
    parent_classes = ['Bootstrap4ListGroupPlugin']

    fieldsets = [
        (None, {
            'fields': (
                'list_context',
                'list_state',
            )
        }),
        (_('Advanced settings'), {
            'classes': ('collapse',),
            'fields': (
                'tag_type',
                'attributes',
            )
        }),
    ]

    def render(self, context, instance, placeholder):
        link_classes = ['list-group-item']
        if instance.list_context:
            link_classes.append('list-group-item-{}'.format(instance.list_context))
        if instance.list_state:
            link_classes.append(instance.list_state)

        classes = concat_classes(link_classes + [
            instance.attributes.get('class'),
        ])
        instance.attributes['class'] = classes

        return super(Bootstrap4ListGroupItemPlugin, self).render(
            context, instance, placeholder
        )


plugin_pool.register_plugin(Bootstrap4ListGroupPlugin)
plugin_pool.register_plugin(Bootstrap4ListGroupItemPlugin)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from functools import partial

from django.db import models
from django.utils.translation import ugettext_lazy as _
from django.utils.translation import ungettext

from cms.models import CMSPlugin

from six import python_2_unicode_compatible

from djangocms_bootstrap4.constants import DEVICE_SIZES
from djangocms_bootstrap4.fields import (
    AttributesField, IntegerRangeField, TagTypeField,
)
from djangocms_bootstrap4.helpers import mark_safe_lazy

from .constants import (
    GRID_COLUMN_ALIGNMENT_CHOICES, GRID_COLUMN_CHOICES, GRID_CONTAINER_CHOICES,
    GRID_ROW_HORIZONTAL_ALIGNMENT_CHOICES, GRID_ROW_VERTICAL_ALIGNMENT_CHOICES,
    GRID_SIZE,
)


@python_2_unicode_compatible
class Bootstrap4GridContainer(CMSPlugin):
    """
    Layout > Grid: "Container" Plugin
    https://getbootstrap.com/docs/4.0/layout/grid/
    """
    container_type = models.CharField(
        verbose_name=_('Container type'),
        choices=GRID_CONTAINER_CHOICES,
        default=GRID_CONTAINER_CHOICES[0][0],
        max_length=255,
        help_text=mark_safe_lazy(_(
            'Defines if the grid should use fixed width (<code>.container</code>) '
            'or fluid width (<code>.container-fluid</code>).'
        )),
    )
    tag_type = TagTypeField()
    attributes = AttributesField()

    def __str__(self):
        return str(self.pk)

    def get_short_description(self):
        text = ''
        for item in GRID_CONTAINER_CHOICES:
            if item[0] == self.container_type:
                text = item[1]
        return '({})'.format(text)


@python_2_unicode_compatible
class Bootstrap4GridRow(CMSPlugin):
    """
    Layout > Grid: "Row" Plugin
    https://getbootstrap.com/docs/4.0/layout/grid/
    """
    vertical_alignment = models.CharField(
        verbose_name=_('Vertical alignment'),
        choices=GRID_ROW_VERTICAL_ALIGNMENT_CHOICES,
        blank=True,
        max_length=255,
        help_text=mark_safe_lazy(_(
            'Read more in the <a href="{link}" target="_blank">documentation</a>.')
                .format(link='https://getbootstrap.com/docs/4.0/layout/grid/#vertical-alignment')
        ),
    )
    horizontal_alignment = models.CharField(
        verbose_name=_('Horizontal alignment'),
        choices=GRID_ROW_HORIZONTAL_ALIGNMENT_CHOICES,
        blank=True,
        max_length=255,
        help_text=mark_safe_lazy(_(
            'Read more in the <a href="{link}" target="_blank">documentation</a>.')
                .format(link='https://getbootstrap.com/docs/4.0/layout/grid/#horizontal-alignment')
        ),
    )
    gutters = models.BooleanField(
        verbose_name=_('Remove gutters'),
        default=False,
        help_text=_('Removes the marginal gutters from the grid.'),
    )
    tag_type = TagTypeField()
    attributes = AttributesField()

    def __str__(self):
        return str(self.pk)

    def get_short_description(self):
        column_count = len(self.child_plugin_instances or [])
        column_count_str = ungettext(
            '(1 column)',
            '(%(count)i columns)',
            column_count
        ) % {'count': column_count}

        return column_count_str


@python_2_unicode_compatible
class Bootstrap4GridColumn(CMSPlugin):
    """
    Layout > Grid: "Column" Plugin
    https://getbootstrap.com/docs/4.0/layout/grid/
    """
    column_type = models.CharField(
        verbose_name=_('Column type'),
        choices=GRID_COLUMN_CHOICES,
        default=GRID_COLUMN_CHOICES[0][0],
        blank=True,
        max_length=255,
    )
    column_alignment = models.CharField(
        verbose_name=_('Alignment'),
        choices=GRID_COLUMN_ALIGNMENT_CHOICES,
        blank=True,
        max_length=255,
    )
    tag_type = TagTypeField()
    attributes = AttributesField()

    def __str__(self):
        return str(self.pk)

    def get_short_description(self):
        text = ''
        classes = self.get_grid_values()
        if self.xs_col:
            text += '(col-{}) '.format(self.xs_col)
        else:
            text += '(auto) '
        if self.column_type != 'col':
            text += '.{} '.format(self.column_type)
        if classes:
            text += '.{}'.format(' .'.join(self.get_grid_values()))
        return text

    def get_grid_values(self):
        classes = []
        for device in DEVICE_SIZES:
            for element in ('col', 'order', 'offset', 'ml', 'mr'):
                size = getattr(self, '{}_{}'.format(device, element))
                if isinstance(size, int) and (element == 'col' or element == 'order' or element == 'offset'):
                    if device == 'xs':
                        classes.append('{}-{}'.format(element, int(size)))
                    else:
                        classes.append('{}-{}-{}'.format(element, device, int(size)))
                elif size:
                    if device == 'xs':
                        classes.append('{}-{}'.format(element, 'auto'))
                    else:
                        classes.append('{}-{}-{}'.format(element, device, 'auto'))

        return classes


IntegerRangeFieldPartial = partial(
    IntegerRangeField,
    blank=True,
    null=True,
    max_value=GRID_SIZE,
)

BooleanFieldPartial = partial(
    models.BooleanField,
    default=False,
)

# Loop through Bootstrap 4 device choices and generate
# model fields to cover col-*, order-*, offset-*, etc.
for size in DEVICE_SIZES:
    # Grid size
    Bootstrap4GridColumn.add_to_class(
        '{}_col'.format(size),
        IntegerRangeFieldPartial(),
    )
    # Grid ordering
    Bootstrap4GridColumn.add_to_class(
        '{}_order'.format(size),
        IntegerRangeFieldPartial(),
    )
    # Grid offset
    Bootstrap4GridColumn.add_to_class(
        '{}_offset'.format(size),
        IntegerRangeFieldPartial(),
    )
    # Grid margin left (ml)
    Bootstrap4GridColumn.add_to_class(
        '{}_ml'.format(size),
        BooleanFieldPartial(),
    )
    # Grid margin right (ml)
    Bootstrap4GridColumn.add_to_class(
        '{}_mr'.format(size),
        BooleanFieldPartial(),
    )



# -*- coding: utf-8 -*-
from cms.api import add_plugin
from cms.test_utils.testcases import CMSTestCase

from djangocms_bootstrap4.contrib.bootstrap4_badge.cms_plugins import (
    Bootstrap4BadgePlugin,
)

from ..fixtures import B4TestFixture


class B4BadgePluginTestCase(B4TestFixture, CMSTestCase):

    def test_plugin(self):
        plugin = add_plugin(
            placeholder=self.placeholder,
            plugin_type=Bootstrap4BadgePlugin.__name__,
            language=self.language,
            badge_text="some text",
        )
        plugin.full_clean()
        self.page.publish(self.language)

        with self.login_user_context(self.superuser):
            response = self.client.get(self.request_url)
        self.assertEqual(response.status_code, 200)
        self.assertContains(
            response,
            '<span class="badge badge-primary">some text</span>',
            html=True,
        )

        # test with pills enabled
        plugin = add_plugin(
            placeholder=self.placeholder,
            plugin_type=Bootstrap4BadgePlugin.__name__,
            language=self.language,
            badge_text="some text",
            badge_pills=True,
        )
        plugin.full_clean()
        self.page.publish(self.language)

        with self.login_user_context(self.superuser):
            response = self.client.get(self.request_url)
        self.assertEqual(response.status_code, 200)
        self.assertContains(
            response,
            '<span class="badge badge-pill badge-primary">some text</span>',
            html=True,
        )
